function gtCopyCorrectUndistortWrapper(parameters)
% GTCOPYCORRECTUNDISTORTWRAPPER  Calculates the number of OAR jobs to launch
%     gtCopyCorrectUndistortWrapper(parameters)
%     -----------------------------------------
%       It creates the table used to coordinate the jobs running in parallel
%
%     Version 002 09-11-2011 by LNervo
%       Add comments and clean function
%
%     Version 001 by AKing
%       It also copies certain files needed early in the preprocessing stage, so
%       they are available for live processing
%
%       Modify to use OAR - try to calculate reasonable number of jobs and wall
%       time.
%
%
%
%     FUNCTIONS SUBMITTED WITH OAR:
%[oar]- gtCopyCorrectUndistortCondor

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% get directory info from parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
sourcedir = parameters.acq.collection_dir;
destdir   = fullfile(parameters.acq.dir, '0_rawdata', parameters.acq.name);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% setup database table
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
gtDBConnect();
filetable=[parameters.acq.name '_filetable'];
mym(['drop table if exists ' filetable]);
mym(['create table if not exists ' filetable ' (filename text, unique key (filename(40)))']);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% copy files needed straight away
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

disp('copying .xml file first to get everything started')
try
    [~, ~] = copyfile(fullfile(sourcedir, '*.xml'), destdir);
catch Mexc
    gtPrintException(Mexc, 'not able to copy .xml first. Error:')
    disp('proceeding with other files')
end

disp('copying dark image first')

% are we moving data from collection to analysis directory?  Or has data
% already been moved once?
% are paths the same?
% what is the source dir name?
[path1, source_dirname, ~] = fileparts(sourcedir);
[path2,              ~, ~] = fileparts(destdir);

if strcmp(path1, path2) || strcmp(source_dirname, 'Orig')
    move_data_flag = false;
else
    move_data_flag = true;
end
fnamein = fullfile(parameters.acq.collection_dir, 'darkend0000.edf');
fnameout = fullfile(parameters.acq.dir, '0_rawdata', parameters.acq.name, 'darkend0000.edf');
fnameout_orig = fullfile(parameters.acq.dir, '0_rawdata', 'Orig', 'darkend0000.edf');
% gtCopy will apply those corrections and undistortions as required
try
    gtCopy(fnamein, fnameout, fnameout_orig, parameters, move_data_flag);
catch Mexc
    gtPrintException(Mexc, 'not able to copy darkend0000.edf first. Error:')
    disp('proceeding with other files')
end

% may want to add other file - pre-scan calibration images?
disp('Can add other things that should be copied first')


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% calculate how many jobs to launch
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%count files in source dir, nfiles
sfiles = dir(fullfile(sourcedir, '*'));
cfiles = dir(fullfile(destdir, '*'));
% This can be quicker and faster:
cfnames = {cfiles.('name')};
% Slower and more tedious:
%for n = 1:length(cfiles)
%    cfnames{n} = cfiles(n).name;
%end

n = 1;
%go through the source list, remove directories, parameters.mat,
%and anything else already copied
while (n <= length(sfiles))
    sfname = sfiles(n).name;
    sfdir = sfiles(n).isdir;
    if sfdir || strcmpi('parameters.mat', sfname) || any(strcmpi(cfnames, sfname))
        sfiles(n) = [];
    else
        n = n+1;
    end
end
nfiles = length(sfiles);

% how many files should this dataset contain?
if strcmpi(parameters.acq.type, '360degree')
    totproj = 2*parameters.acq.nproj;
elseif strcmpi(parameters.acq.type, '180degree')
    totproj = parameters.acq.nproj;
else
    Mexc = MException('ACQ:invalid_parameter', ...
                      'Unknown type of scan. Check parameters.acq.type!');
    throw(Mexc);
end
totfiles = totproj+(((totproj/parameters.acq.refon)+1)*parameters.acq.nref)+parameters.acq.ndark;
disp(['Number ' num2str(totfiles) ' files to be copied...'])

% for OAR - want either several jobs for a short time, or a few jobs for
% the time taken to acquire the dataset
if isfield(parameters.acq, 'count_time')
    tottime = totfiles*(parameters.acq.count_time+0.4)*1.3; %1.3 safety factor
    % worst case E2V readout time
else
    tottime = totfiles*5;
end

if ( nfiles > (totfiles*0.6) )
    % finished - or nearly finished
    OARjobs = ceil(totfiles/500);
    OARtime = (totfiles/OARjobs)*3; % guess 3 seconds per job
else
    % running - need to run until end of scan
    OARjobs = 5;
    OARtime = tottime;
end
OARtmp   = ceil(OARtime/60); % minutes
OARm     = mod(OARtmp, 60);  % minutes
OARh     = (OARtmp-OARm)/60; % hours
walltime = sprintf('%02d:%02d:00', OARh, OARm);

OAR_parameters.walltime = '2:00:00';
fprintf('hard coded walltime to 2 hours instead of the estimated %s hours...\n',walltime)

fprintf('Number of OAR jobs: %d\n', OARjobs);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% launch OAR jobs
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% OAR_make(executable, first, last, njobs, otherparameters, submitflag, varargin)
OAR_make('gtCopyCorrectUndistortCondor', 0, totfiles, OARjobs, parameters.acq.dir, true, 'walltime', OAR_parameters.walltime);


end % end function
