classdef GtThreshold < handle
    properties
        conf;

        % Reconstruction parameters from parameters.mat
        param;
    end
    methods
        function obj = GtThreshold(varargin)
            obj.conf = [];
            obj.conf.verbose = true;

            obj.param = load('parameters.mat');
            obj.param = obj.param.parameters;

            [obj.param.rec, rej_params] = parse_pv_pairs(obj.param.rec, varargin);
            if (~isfield(obj.param.rec, 'do_morph_recon'))
                obj.param.rec.do_morph_recon = false;
            end

            obj.conf = parse_pv_pairs(obj.conf, rej_params);
        end

        function [defaults, sample] = generateDefaults(obj)
            samplefile  = fullfile(obj.param.acq.dir, '4_grains', 'sample.mat');
            sample      = GtSample.loadFromFile(samplefile);

            defaults = [];
            defaults.percentile = obj.param.rec.percentile2;

            defaults.phases = {};
            for phaseID = 1:length(sample.phases)
                defaults.phases{phaseID}.min_id = 1;
                defaults.phases{phaseID}.max_id = length(sample.phases{phaseID}.grains);
            end
        end

        function batchThreshold(obj, phaseID, firstID, lastID, percentile)
        % GTTHRESHOLD/BATCHTHRESHOLD

            [defaults, sample] = obj.generateDefaults();

            if (exist('percentile', 'var'))
                obj.param.rec.percentile2 = percentile;

                parameters = obj.param;
                save('parameters.mat', 'parameters');
            end

            % Here is for the lazy who wants to launch from CMD line.
            if (ischar(firstID) && strcmpi(firstID, 'first'))
                firstID = defaults.phases{phaseID}.min_id;
            end
            if (ischar(lastID) && strcmpi(lastID, 'last'))
                lastID = defaults.phases{phaseID}.max_id;
            end

            if ((firstID < defaults.phases{phaseID}.min_id) ...
                    || (firstID > lastID) ...
                    || (lastID > defaults.phases{phaseID}.max_id))
                error('SEGMENT:wrong_argument', ...
                      'The range indexes are not consistent: (%d, %d).', ...
                      firstID, lastID);
            end

            phaseDir = fullfile(parameters.acq.dir, '4_grains', ...
                                sprintf('phase_%02d', phaseID));

            if (~obj.conf.verbose), fprintf('Phase %02d: ', phaseID); end
            for grainID = firstID : lastID
                if (~isdeployed), fprintf('%04d/(%04d..%04d)', grainID, firstID, lastID); end

                grain_file = fullfile(phaseDir, sprintf('grain_%04d.mat', grainID));
                grain      = load(grain_file, 'vol', 'shift', 'id');
                grain      = obj.thresholdAutoSingleGrain(grain);
                seg        = grain.seg;
                segbb      = grain.segbb;
                threshold  = grain.threshold;
                Area       = grain.Area;
                sample.phases{phaseID}.setThreshold(grainID, threshold);
                sample.phases{phaseID}.setBoundingBox(grainID, segbb);

                save(grain_file, '-append', 'seg', 'segbb', 'threshold', 'Area');

                if (~obj.conf.verbose), fprintf('\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b'); end
            end
            if (~obj.conf.verbose), fprintf('(%04d..%04d) Done.\n', firstID, lastID); end
            filetable = [parameters.acq.name '_filetable'];
            sample.mergeToFile(filetable, samplefile, 'sample');
        end

        function grain = thresholdAutoSingleGrain(obj, grain)
        % GTTHRESHOLD/THRESHOLDAUTOSINGLEGRAIN Segment 3D grain volume
        %
        % function grain = thresh.thresholdAutoSingleGrain(grain, rec)
        %
        % INPUTS:  grain.vol          graylevel reconstruction of grain volume
        %          rec                contents of parameters.rec structure:
        %             .percentile2    adaptive threshold see below
        %             .do_morph_recon remove not connected areas by morphological
        %                             reconstruction (<logical> {true})
        % OUTPUTS:
        %          grain.thresh Threshold value
        %          grain.seg    Segmented grain volume
        %          grain.Area   Area of grain
        %          grain.segbb  BoundingBox (gt convention) for insertion of seg
        %                       volume into sample volume
        %
        % explanation - we expect a histogram with a peak corresponding to the
        % grain, and a peak a low values corresponding to the background of zeros.
        % We want a threshold value below the peak corresponding to the grain, but
        % above the background peak.

            vol = grain.vol;

            % calulate intensity in the central region of the grain
            centerx = round(size(vol,1) * 0.45 : size(vol,1) * 0.55);
            centery = round(size(vol,2) * 0.45 : size(vol,2) * 0.55);
            centerz = round(size(vol,3) * 0.45 : size(vol,3) * 0.55);

            background = gtImgMeanValue(vol(centerx, centery, centerz)) / 10;
            % Small performance boost
            indexes = vol > background;
            % define threshold as percentile/100 x standard dev below the mean,
            % excluding low (10 percentile background) values
            threshold_val = mean( vol(indexes) ) ...
                            - (obj.param.rec.percentile2 / 100) * std( vol(indexes) );

            grain = obj.segmentAndDoMorph( grain, threshold_val, ...
                                            obj.param.rec.do_morph_recon, ...
                                            [centerx centery centerz]);
        end

        function grain = thresholdSingleGrain(obj, grain, threshold, center)
        % GTTHRESHOLD/THRESHOLDSINGLEGRAIN

            if (exist('center', 'var') && all(center > 0))
                do_morpho = true;
            else
                center = zeros(1, 3);
                do_morpho = false;
                warning('THRESHOLD:less_capability', ...
                        'Morphological reconstruction is disabled')
            end

            grain = obj.segmentAndDoMorph(grain, threshold, do_morpho, center);
        end

        function segvol = thresholdVolume(obj, volume, threshold, center)
        % GTTHRESHOLD/THRESHOLDVOLUME

            if (exist('center', 'var') && all(center > 0))
                do_morpho = true;
            else
                center = zeros(1, 3);
                do_morpho = false;
                warning('THRESHOLD:less_capability', ...
                        'Morphological reconstruction is disabled')
            end

            mess = 'Error while segmenting the volume';
            segvol = obj.segmentVol(volume, threshold, do_morpho, mess, center);
        end

        function grain = thresholdSingleGrainInFile(obj, phaseID, grainID, threshold, center)
        % GTTHRESHOLD/THRESHOLDSINGLEGRAININFILE

            fprintf('Segmenting Grain %d, from phase %d.. ', grainID, phaseID);
            filePath = fullfile('4_grains', sprintf('phase_%02d', phaseID), ...
                                sprintf('grain_%04d.mat', grainID));
            grain = load(filePath);

            if (exist('center', 'var'))
                grain = thresholdSingleGrain(obj, grain, threshold, center);
            else
                grain = thresholdSingleGrain(obj, grain, threshold);
            end

            seg        = grain.seg;
            segbb      = grain.segbb;
            threshold  = grain.threshold;
            Area       = grain.Area;

            save(filePath, '-append', 'seg', 'segbb', 'threshold', 'Area');
            fprintf('Done.\n');
        end
    end

    methods (Access = protected)
        function grain = segmentAndDoMorph(obj, grain, threshold, do_morpho, center)
        % GTTHRESHOLD/SEGMENTANDDOMORPH

            message = sprintf(['morphological reconstruction failed for ' ...
                               'grain %d - continue without...'], ...
                              grain.id);
            segvol = obj.segmentVol(grain.vol, threshold, do_morpho, message, ...
                                    center);

            regprop = regionprops(segvol, 'BoundingBox', 'Area');
            if (length(regprop) > 1)
                % let's take the biggest! (usually happens without morpho)
                regprop = regprop(1);
            end
            segbb   = regprop.BoundingBox([2 1 3 5 4 6]);

            % ceil because Matlab regionprops yields "0.5" (instead of 1) for bb
            % start values
            grain.seg       = gtCrop(segvol, ceil(segbb));
            grain.Area      = regprop.Area;
            segbb(1:3)      = grain.shift + floor(segbb(1:3));
            grain.segbb     = segbb;
            grain.threshold = threshold;
        end

        function segvol = segmentVol(~, vol, threshold, do_morpho, errorMsg, center)
        % GTTHRESHOLD/SEGMENTVOL

            segvol = vol > threshold;

            % morphological reconstruction removes not connected areas
            if (do_morpho && all(center > 0))
                try
                    marker = false(size(vol));
                    marker(center(1), center(2), center(3)) = true;
                    marker = marker & segvol;
                    segvol = double(imreconstruct(marker, segvol));
                catch mexc
                    gtPrintException(mexc, errorMsg);
                end
            end
        end
    end
end
