function [output, redo] = gtAssembleVol3D(varargin)
% [output, redo] = gtAssembleVol3D(varargin)
%
%   output - volume the size of the sample
%   add some useful info to the mat file - bounding box and CoM of postprocessed grain
%
%   use parse_pv_pairs to handle various optional arguments, which:
%                 savefilename = string
%                     overlaps = 'zero' / 'summed'
%              add_to_filename = .edf volume containing a set of existing grains
%       display_error_messages = logical
%                       grains = Cell array containing the pre-loaded grain
%                                volumes
%             update_status_cb = Callback to update the status of the
%                                computation (used for instance in a gui). The
%                                said callback is executed every time the loop
%                                terminates the processing of an entry.
%
%   not designed to be used compiled
%

% load parameters and grain
load('parameters.mat');
load('grain.mat');

% handle variables
par.savefilename = [];              % filename to save volume
par.overlaps = 'summed';            % how to handle overlapping grains ('zero' / 'summed')
par.add_to_filename = [];           % to add grains to a preexisting volume (saved as edf)
par.list = [];                      % if list is 'all', or [], load all grains.  Otherwise, only the list
par.display_error_messages = true;  % show error messages
par.grains = [];                    % Pre-loaded grain volumes
par.update_status_cb = [];          % Callback to update the status of anything

par = parse_pv_pairs(par,varargin);

redo=[];                            % Keeps track of the grains that had an error

overlap_to_zero = strcmpi(par.overlaps,'zeros') || strcmpi(par.overlaps,'zero');

% if we are adding grains to a file that has already been created
if ~isempty(par.add_to_filename)
    % look for the file
    absolute_file_path = fullfile(parameters.acq.dir, '5_reconstruction', par.add_to_filename);

    if exist(par.add_to_filename, 'file')
        output = edf_read(par.add_to_filename);
        % check if it's in the 5_reconstruction directory
    elseif exist( absolute_file_path, 'file')
        output = edf_read(absolute_file_path);
        % otherwise, complain
    else
        fprintf('Cannot find the volume to add to: %s\n', par.add_to_filename)
        disp('check the filename / path')
        disp('quitting')
        return
    end
else
    % otherwise, start with zeros
    output = zeros(parameters.acq.bb(3), parameters.acq.bb(3),parameters.acq.bb(4));
end

% handle list
if isempty(par.list) || strcmpi(par.list, 'all')
    list = 1:length(grain);
else
    list = par.list;
end

% if we want to add grains to an existing volume, this is loaded in.  Then
% labels in the list are set to zero, before the grains are "re-added" to the volume
% I don't think this will deal with overlapping grains correctly.
if (max(output(:)) > 0)
    for ii = 1:length(list)
        output(output == list(ii)) = 0;
    end
end

% if not saving data, give a warning
if isempty(par.savefilename)
    disp('not saving data - to save, supply a filename')
    pause(2)
end

% not that "zeros" is ambiguous in Matlab because it is a function, and in
% this context we normally use it as a string.  Use "zero" instead, small
% bodge here in case this is not done.
if par.overlaps == 0
    par.overlaps='zero';
    disp('guessing that overlaps should be set to zero')
end

if ~isempty(par.grains) && (length(par.grains) ~= length(grain))
    Mexc = MException('ASSEMBLE:wrong_grain_numbers', ...
                      [ 'The number of the provided grain volumes, and the ' ...
                        'list of grains in the grain.mat file differ']);
    throw(Mexc)
end

for k = 1:length(list)

    ii = list(k);

    fprintf('~~~~~~~~  doing grain %d  ~~~~~~~~~\n', ii)
    try
        % Directory of the grain ii
        graindir = fullfile('4_grains', sprintf('grain%d_', ii));

        mat_filename = fullfile(graindir, sprintf('grain%d_combined.mat',ii) );
        if exist(mat_filename,'file')
            tmp = load(mat_filename, 'bad', 'z1', 'nz', 'x1','y1','nx','ny');
        else
            mat_filename = fullfile(graindir, sprintf('grain%d_.mat',ii) );
            tmp = load(mat_filename, 'bad', 'z1', 'nz', 'x1','y1','nx','ny');
        end

        %  1=bad / 2=combined / 0.5=bad-ish / 3= impose grainID for the region occupied by this grain
        if isfield(tmp, 'bad') && (tmp.bad==1 || tmp.bad==2)
            fprintf('skipping grain %d\n', ii)
            continue
        end %bads check if bad skip/else try to do grain

        if (isempty(par.grains))
            vol_filename = fullfile(graindir, sprintf('grain%d_filled.edf',ii) );
            if exist(vol_filename,'file')  %if post-processed file exists
                vol = edf_read(vol_filename);
            else
                grain_name   = fullfile(graindir, 'grain.edf' );
                vol = edf_read(grain_name);
            end % if exist vol_filename
        else
            vol = par.grains(ii);
        end

        [min1,min2,min3]=gtTrARTToSamBB3D(tmp.x1+tmp.nx-1, tmp.y1, tmp.z1, parameters);
        [max1,max2,max3]=gtTrARTToSamBB3D(tmp.x1, tmp.y1+tmp.ny-1, tmp.z1+tmp.nz-1, parameters);

        % have to deal with bounding boxs that exceed the slice size -
        % _clipping_ of exceeding volum from volume sample
        if (min1 < 1)
            temp = 2-min1; %if min of first index==0, crop one row of voxels from vol
            min1 = 1;
            vol = vol(temp:end,:,:);
        end
        if (min2 < 1)
            temp = 2-min2; %if min of second index==0, crop one row of voxels from vol
            min2 = 1;
            vol = vol(:,temp:end,:);
        end
        if (min3 < 1)
            temp = 2-min3; %if z1==0, crop one row of voxels from vol
            min3 = 1;
            vol = vol(:,:,temp:end);
        end
        if (max1 > parameters.acq.bb(3))
            temp = max1-parameters.acq.bb(3);%if x2 is one too big, crop one row of voxels
            max1 = parameters.acq.bb(3);
            vol = vol(1:end-temp,:,:);
        end
        if (max2 > parameters.acq.bb(3))
            temp = max2-parameters.acq.bb(3);%if y2 is one too big, crop one row of voxels
            max2 = parameters.acq.bb(3);
            vol = vol(:,1:end-temp,:);
        end
        if (max3 > parameters.acq.bb(4))
            temp = max3-parameters.acq.bb(4);%if z2 is one too big, crop one row of voxels
            max3 = parameters.acq.bb(4);
            vol = vol(:,:,1:end-temp);
        end

        %add grain to volume, colour coded
        %vol = vol*ii;

        if overlap_to_zero
            % stop overlapping...
            % Matlab sugars to do the same that 'internal_gtAssignGrainToVol_interf'
            % does: (please keep this around, in case you break the C function)
%             tmp_output = zeros(size(output));
%             tmp_output(min1:max1, min2:max2, min3:max3) = vol;
%             output(find(output == 0 & tmp_output)) = ii; %set undisputed voxels to i
%             output(find(output ~= ii & tmp_output)) = -1; %set overlaps to -1
            output = internal_gtAssignGrainToVol_interf(output, vol, ii, [min1 max1 min2 max2 min3 max3]);
        else
            % Matlab sugar to do the same that 'internal_gtAssignGrainToVol'
            % does: (please keep this around, in case you break the C function)
%             output(min1:max1, min2:max2, min3:max3) = output(min1:max1, min2:max2, min3:max3) + vol;
            output = internal_gtAssignGrainToVol(output, vol, ii, [min1 max1 min2 max2 min3 max3]);
        end
    catch Mexc
        gtPrintException(Mexc, sprintf('Grain %d failed', ii));
        redo = [redo ii];
    end %try catch loop

    gtTriggerUpdateCallback(par.update_status_cb);
end  % for loop through all grains

if overlap_to_zero
    output(output == -1) = 0; %set overlapping regions to zero - this could be another label colour
end

% did any grains fail?
if ~isempty(redo)
    for ii=1:length(redo)
        fprintf('grain%d failed for some reason!\n', redo(ii))
    end
end

% save the output
if ~isempty(par.savefilename)
    name = fullfile('5_reconstruction', par.savefilename);
    fprintf('writing volume %s\n',name);
    edf_write(output,name,'uint16');
end

end
