function gtSetupReconstruction(phaseID)
% GTSETUPRECONSTRUCTION  Simple helper function to launch reconstruction and segmentation of
% absorption volume - as well as reconstruction and segmentation of grain volumes
%
%     gtSetupReconstruction(phaseID)
%     ------------------------------
%
%     INPUT:
%       phaseID = number of the phase <double> {1}
%
%     Version 003 29-05-2012 by LNervo
%       Add comments in header about functions used
%       Add possibility to recompile 'gtChangeThreshold3D'
%
%     Version 002 25-05-2012 by LNervo
%       Commenting and formatting: fprintf -> disp; y/n -> [y/n];
%
%
%     FUNCTIONS COMPILED:
%[bin]- gtChangeThreshold3D
%
%     FUNCTIONS SUBMITTED TO OAR:
%[oar]- gtChangeThreshold3D


GPU_OK = gtCheckGpu();

if ~GPU_OK
    disp('Not on a GPU machine, please launch reconstruction from a GPU machine !');
    disp('from rnice: oarsub -I -l "walltime=hh:mm:ss" -p "gpu = ''YES''"')
    return;
end

if ~exist('phaseID','var') || isempty(phaseID)
    phaseID = 1;
end

check = inputwdefault('Do you want to recompile the function ''gtChangeThreshold3D''? [y/n]','n');
if strcmpi(check,'y')
    gt_mcc('gtChangeThreshold3D');
end

parameters = [];
load('parameters.mat');
parameters_savefile = fullfile(parameters.acq.dir, 'parameters.mat');

% Let's check every field is in, and in case let's update it!
parameters.('rec') = gtCheckParameters(parameters.('rec'), gtRecDefaultParameters());
save(parameters_savefile);

nof_phases = length(parameters.cryst);

if nof_phases > 1
    phaseID = inputwdefaultnumeric('Enter phaseID', num2str(1));
end
phaseID_str = sprintf('%02d',phaseID);


disp(['Setting up Reconstruction for phase ' phaseID_str ' (' parameters.cryst(phaseID).name ')']);

filetable = [parameters.acq.name '_filetable'];
samplefile = fullfile(parameters.acq.dir, '4_grains', 'sample.mat');
if exist(samplefile, 'file')
   sample = GtSample.loadFromFile(samplefile);
else
   disp('Can not find 4_grains/sample.mat - have you run gtSetupForwardSimulation ?');
   exit;
end

% add the possibility to redo abs reconstruction, even if existing
absorption_volume = fullfile(parameters.acq.dir, '5_reconstruction', 'volume_absorption.mat');
if ~exist(absorption_volume,'file')
    check = inputwdefault('Can not find absorption volume - would you like to launch reconstruction of absorption volume ? [y/n]', 'y');
    if strcmpi(check, 'y')
        abs = gtAstraAbsorptionVolume(parameters);
        save(absorption_volume, 'abs');
        sample.absVolFile = fullfile('5_reconstruction', 'volume_absorption.mat');
        save(samplefile, 'sample');
    end
else
    check = inputwdefault('Absorption volume found - would you like to overwrite it? [y/n]', 'n');
    if strcmpi(check, 'y')
        abs = gtAstraAbsorptionVolume(parameters);
        save(absorption_volume, 'abs');
        sample.absVolFile = fullfile('5_reconstruction', 'volume_absorption.mat');
        save(samplefile, 'sample');
    else
        abs = load(absorption_volume, 'abs');
        abs = abs.abs;
    end
end

% add the possibility to redo volume threshold, even if existing
absorption_mask = fullfile(parameters.acq.dir, '5_reconstruction', 'volume_mask.mat');
if ~exist(absorption_mask,'file')
    check = inputwdefault('Can not find absorption mask - would you like to segment absorption volume ? [y/n]', 'y');
    if strcmpi(check, 'y')
        disp('Please create file volume_mask.mat from absorption volume (right click in the volume to select a seed point and then press ''Segment'')');
        GtGuiThresholdVolume(abs);
    end
else
    check = inputwdefault('Absorption mask found - would you like to overwrite it? [y/n]', 'n');
    if strcmpi(check, 'y')
        disp('right click in the volume to select a seed point and then press ''Segment''');
        GtGuiThresholdVolume(abs);
    end
end

% get reconstruction parameters
check = inputwdefault('Use default parameters for grain reconstruction ? [y/n]', 'y');
if ~strcmpi(check, 'y')
    list = build_list_v2();
    parameters.rec = gtModifyStructure(parameters.rec, list.rec(:,1:2));
    save(parameters_savefile,'parameters');
end

first = 1;
last  = length(sample.phases{phaseID}.grains);

check = inputwdefault('Launch reconstruction? [y/n]', 'y');
if  strcmpi(check, 'y')
    gtReconstructGrains(first, last, parameters.acq.dir, phaseID, parameters);
end

check = inputwdefault('Launch grain segmentation? [y/n]', 'y');
if  strcmpi(check, 'y')
    check = inputwdefault('Launch grain segmentation via OAR ? [y/n]', 'y');
    if  strcmpi(check, 'y')
        disp('Launching several separate OAR jobs')

        oar.njobs    = max(ceil(last/30), 30); % max 30 jobs
        oar.walltime = last / oar.njobs * 300; % allow up to 300 seconds per grain

        OARtmp = ceil(oar.walltime / 60); % minutes
        OARm   = mod(OARtmp, 60); % minutes
        OARh   = (OARtmp - OARm) / 60; % hours
        OAR_parameters.walltime = sprintf('%02d:%02d:00',OARh, OARm);
        clear OARtmp OARm OARh oar.walltime

        disp('Using OAR parameters:')
        disp(OAR_parameters)
        disp(['Using ' num2str(oar.njobs) ' OAR jobs']);
        pause(0.5);
        OAR_make('gtChangeThreshold3D',first, last, oar.njobs, [parameters.acq.dir ' ' phaseID_str], true, 'walltime', OAR_parameters.walltime);
    else
        gtChangeThreshold3D(first, last, parameters.acq.dir, phaseID);
    end
end

end % end of function

