function sample = gtSolveGrainsConflicts(minnum, save_flag)
% GTSOLVEGRAINSCONFLICTS
%     sample = gtSolveGrainsConflicts(minnum, save_flag)
%     --------------------------------------------------
%     To be run in the analysis directory after gtUpdateGrainsConflicts
%
%     Reads ./4_grains/sample.mat
%           ./4_grains/spot2grain.mat
%
%     Writes ./4_grains/out.mat
%        out{phaseid}.deselected_grains     = deselected grains
%        out{phaseid}.modified_grains       = modified grains with summary{grainid}.totspots < minnum
%        out{phaseid}.modified_difspots     = corresponding difspots deselected from those grains
%        out{phaseid}.conflicts             = grainid still in conflicts
%        out{phaseid}.conflicts_list        = grainid still in conflicts
%        out{phaseid}.conflicts_unique      = grainid still in conflicts
%        out{phaseid}.spot2grain            = {spot2grain, spot2phase};
%
%     INPUT:
%       minnum    = minimum number of spots to be discarded <double> {4}
%       save_flag = flag to save 4_grains/sample.mat and 4_grains/spot2grain.mat
%                   <logical> {false}
%
%     OUTPUT:
%       sample = updated sample.mat <struct>
%
%     Version 003 15-06-2012 by LNervo
%       sample.phases{phaseid}.summary{grainid}.flag can be:
%          0 - no conflict
%          1 - conflict and threshold < 0
%          2 - conflict and boundingBox = [0 0 0 0 0 0]
%          3 - conflict and totspots < minnum


if ~exist('minnum','var') || isempty(minnum)
    minnum = 4;
end
if ~exist('save_flag','var') || isempty(save_flag)
    save_flag = false;
end

out        = [];
sample     = [];
spot2grain = [];
spot2phase = [];


bkpdir  = fullfile('4_grains','archive');
fname0  = fullfile(bkpdir,'%s');

sample_root = fullfile('4_grains','sample.mat');
sampleFile = gtLastFileName(sprintf(fname0,['sample_' date() '_']),'new');

spot_root   = fullfile('4_grains','spot2grain.mat');
spotFile   = gtLastFileName(sprintf(fname0,['spot2grain_' date() '_']),'new');

out_root    = fullfile('4_grains','out.mat');
outFile    = gtLastFileName(sprintf(fname0,['out_' date() '_']),'new');

logName    = strrep(outFile,'.mat','.log');

disp(['Log filename in: ' logName])
diary( logName )

load(sample_root);
load(spot_root);
num_phases = length(sample.phases);

difspots_list = [];


for phaseid = 1 : num_phases

    redo_rec      = false;
    reco_assemble = false;

    phase = sample.phases{phaseid};
    fprintf('*************************\n')
    fprintf('***     PHASE: %02d     ***\n',phaseid)
    fprintf('*************************\n')

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % get grains with threshold < 0
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    grainIDs_thr = phase.getGrainsWithFieldValueLessThan('threshold', 0);
    if ~isempty(grainIDs_thr)
        % set phase.selectedGrains = false
        arrayfun(@(array) phase.setSelected(array, false), grainIDs_thr);
        % set summary.flag = 1
        arrayfun(@(array) phase.setFlag(array, 1), grainIDs_thr);
        disp(['Deselected ' num2str(length(grainIDs_thr)) ' grains with threshold < 0: '])
        disp(grainIDs_thr)
        disp('<<< Check the grain segmentation for those grains >>>')
        redo_rec = false;
        reco_assemble = true;
    end

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % bbox empty
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    grainIDs_bbox0  = find(arrayfun(@(num) ~isempty(intersect(phase.getBoundingBox(num),[0 0 0 0 0 0], 'rows')), 1:length(phase.grains)));
    if ~isempty(grainIDs_bbox0)
        % set phase.selectedGrains = false
        arrayfun(@(array) phase.setSelected(array, false), grainIDs_bbox0);
        % set summary.flag = 2
        arrayfun(@(array) phase.setFlag(array, 2), grainIDs_bbox0);
        disp(['Deselected ' num2str(length(grainIDs_bbox0)) ' grains with boundingBox = [0 0 0 0 0 0]: '])
        disp(grainIDs_bbox0)
        disp('<<< Check grain segmentation for those grains >>>')
        redo_rec = false;
        reco_assemble = true;
    end


    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    grainIDs_thr0 = phase.getGrainsWithFieldValueEqualsTo('threshold', 0);
    if ~isempty(grainIDs_thr0)
        disp(['There are ' num2str(length(grainIDs_thr0)) ' grains with threshold = 0.'])
        if length(grainIDs_thr0) == length(phase.grains)
            disp('  Probably you did not run gtSetupReconstruction yet!')
        else
            disp(grainIDs_thr0)
            disp('<<< Run again gtReconstructGrain and gtChangeThreshold3D for those grains >>>')
            redo_rec = true;
            redo_assemble = true;
        end
    end

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % get grains with (completeness - mean ) < 3*std
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    thr_com = mean(phase.completeness(:))-3*std(phase.completeness(:));
    grainIDs_com = phase.getGrainsWithCompletenessLessThan(thr_com);
    if ~isempty(grainIDs_com)
        disp(['  Mean(completeness): ' num2str(mean(phase.completeness(:)))])
        disp(['Found ' num2str(length(grainIDs_com)) ' grains with completeness < ' num2str(thr_com) ' ( = mean - 3*std ): '])
        disp(grainIDs_com)
        disp('Please check the goodness of those grains manually.')
    end

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % get deselected grains
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    grainIDs_desel = phase.getDeselectedGrains();
    if ~isempty(grainIDs_desel)
        % set summary.delete = true
        arrayfun(@(array) phase.setDelete(array, true), grainIDs_desel);
        disp(['In total deselected ' num2str(length(grainIDs_desel)) ' grains: '])
        disp(grainIDs_desel)
    end

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % deselect difspots in conflict from grains{grainid} still in conflicts
    % if difspots <= minnum
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    grainIDs = phase.getHasConflictsTrue();
    if ~isempty(grainIDs)
        disp(['There are still ' num2str(length(grainIDs)) ' grains with conflict: '])
        disp(grainIDs)
    end

    % difspots < minnum
    grainIDs_min = phase.getGrainsWithNofConflictsLessThan(minnum);
    if ~isempty(grainIDs_min)
        % get all the grains.selectedDiffspots
        disp(['Found ' num2str(length(grainIDs_min)) ' grains with less than ' num2str(minnum) ' difspots in conflict: '])
        disp(grainIDs_min)

        [~] = arrayfun(@(array) phase.buildIdSpot(array), grainIDs_min, 'UniformOutput', false);
        % get indexes for each grain with difspots in conflict
        indexes_min = arrayfun(@(array) phase.getIdSpotIndexes(array), grainIDs_min, 'UniformOutput', false);
        % get ids for each grain with difspots in conflict
        idspots_min = arrayfun(@(array) phase.getIdSpotIds(array), grainIDs_min, 'UniformOutput', false);

        % set to false in grains{grainid}.selectedDiffspots
        arrayfun(@(num) phase.setSelectedDiffspotsIndexes(grainIDs_min(num), indexes_min{num}, false), 1:length(grainIDs_min));
        disp(['Deselected the corresponding diffspots in conflict for ' num2str(length(grainIDs_min)) ' grains:'])
        arrayfun(@(num) disp(['  Deselected ' num2str(length(indexes_min{num})) ...
                              ' difspots from grain ' num2str(grainIDs_min(num)) ...
                              ': ' num2str(idspots_min{num})]), 1:length(grainIDs_min));
        % set summary.flag = 3
        arrayfun(@(array) phase.setFlag(array, 3), grainIDs_min);
        % reset summary.delete = false, keeping true for deselected grains
        arrayfun(@(array) phase.setDelete(array, false), grainIDs_min);
        arrayfun(@(array) phase.setDelete(array, true), intersect(grainIDs_min,grainIDs_desel));

        % update spot2grain and spot2phase
        output = sfUpdateSpot2Grain(grainIDs_min, [idspots_min{:}], spot2grain, spot2phase);
        spot2grain = output.spot2grain;
        spot2phase = output.spot2phase;

        if save_flag
            copyfile(spot_root, spotFile);
            save(spot_root,'-struct','output');
            disp(['Updated ' spot_root])
        else
            disp('spot2grain.mat has not been saved on disk.')
        end



        % update summary{grainIDs_min}
        totspots_min = arrayfun(@(num) phase.getTotSpots(grainIDs_min(num)) - length(indexes_min{num}), 1:length(grainIDs_min), 'UniformOutput', false);
        totspots_min = [totspots_min{:}];
        hasSpots_min = totspots_min ~= 0;



        if all(~hasSpots_min)
            arrayfun(@(num) phase.setTotSpots(grainIDs_min(num), []), 1:length(grainIDs_min));
            arrayfun(@(num) phase.setIdSpot(grainIDs_min(num), []), 1:length(grainIDs_min));
            arrayfun(@(num) phase.setNumGrain(grainIDs_min(num), []), 1:length(grainIDs_min));
            arrayfun(@(num) phase.setIdConflict(grainIDs_min(num), []), 1:length(grainIDs_min));

            arrayfun(@(num) phase.setHasConflicts(grainIDs_min(num), false), 1:length(grainIDs_min));
            arrayfun(@(num) phase.setNofConflicts(grainIDs_min(num), 0), 1:length(grainIDs_min));

            arrayfun(@(num) phase.resetConflicts(grainIDs_min(num), 0), 1:length(grainIDs_min));
            disp('Reset conflicts for grains number: ')
            disp(grainIDs_min(~hasSpots_min))
        else
            disp('Grains with still some conflict: ')
            disp(grainIDs_min(hasSpots_min))
            % get existing conflicts
            grainIDs_update = grainIDs_min(hasSpots_min);
            totspots_update = totspots_min(hasSpots_min);
            disp('Update phase info:')
            arrayfun(@(num) phase.setNofConflicts(grainIDs_update(num), totspots_update(num)), 1:length(grainIDs_update));

            arrayfun(@(num) phase.setHasConflicts(num, true), grainIDs_min(hasSpots_min));
            arrayfun(@(num) phase.setHasConflicts(num, false), grainIDs_min(~hasSpots_min));

        end % to update conflicts and summary

        disp('Handles conflicts of modified grains')
        % for each modified grain with conflict, get idconflict
        idconflict_min = arrayfun(@(array) phase.getIdConflict(array), grainIDs_min, 'UniformOutput', false);
        % get only grainids which have less than minnum difspots
        [idconflict_min, ~] = setdiff([idconflict_min{:}],grainIDs_min);
        if ~isempty(idconflict_min)
            [~] = arrayfun(@(array) phase.buildIdSpot(array), idconflict_min, 'UniformOutput', false);

            % get indexes for each grain in conflict with grainIDs_max with
            % less than minnum difspots
            idspots_c_min = arrayfun(@(array) phase.getIdSpotIds(array), idconflict_min, 'UniformOutput', false);
            indexes_c_min = arrayfun(@(array) phase.getIdSpotIndexes(array), idconflict_min, 'UniformOutput', false);
            % set to false in grains{grainid}.selectedDiffspots
            arrayfun(@(num) phase.setSelectedDiffspotsIndexes(idconflict_min(num), indexes_c_min{num}, false), 1:length(indexes_c_min));
            disp(['  Deselected the corresponding diffspots in conflict for ' num2str(length(idconflict_min)) ' conflicts:'])
            arrayfun(@(num) disp(['    Deselected ' num2str(length(indexes_c_min{num})) ...
                                  ' difspots from conflict ' num2str(idconflict_min(num)) ...
                                  ': ' num2str(idspots_c_min{num})]), 1:length(indexes_c_min));
            % set summary.flag = 3
            arrayfun(@(array) phase.setFlag(array, 3), idconflict_min);
            % reset summary.delete = false, keeping true for deselected grains
            arrayfun(@(array) phase.setDelete(array, false), idconflict_min);
            arrayfun(@(array) phase.setDelete(array, true), intersect(idconflict_min,grainIDs_desel));

            % update spot2grain and spot2phase
            output = sfUpdateSpot2Grain(idconflict_min, [idspots_c_min{:}], spot2grain, spot2phase);
            spot2grain = output.spot2grain;
            spot2phase = output.spot2phase;

            if save_flag
                copyfile(spot_root, spotFile);
                save(spot_root,'-struct','output');
                disp(['Updated ' spot_root])
            else
                disp('spot2grain.mat has not been saved')
            end


            % update summary{idconflict_min}
            totspots_c_min = arrayfun(@(num) phase.getTotSpots(idconflict_min(num)) - length(indexes_c_min{num}), 1:length(idconflict_min));
            hasSpots_c_min = totspots_c_min ~= 0;

            if all(~hasSpots_c_min)
                arrayfun(@(num) phase.setTotSpots(idconflict_min(num), []), 1:length(idconflict_min));
                arrayfun(@(num) phase.setIdSpot(idconflict_min(num), []), 1:length(idconflict_min));
                arrayfun(@(num) phase.setNumGrain(idconflict_min(num), []), 1:length(idconflict_min));
                arrayfun(@(num) phase.setIdConflict(idconflict_min(num), []), 1:length(idconflict_min));

                arrayfun(@(num) phase.setHasConflicts(idconflict_min(num), false), 1:length(idconflict_min));
                arrayfun(@(num) phase.setNofConflicts(idconflict_min(num), 0), 1:length(idconflict_min));

                arrayfun(@(num) phase.resetConflicts(idconflict_min(num), 0), 1:length(idconflict_min));
            else
                disp('  Grains with still some conflict: ')
                disp(idconflict_min(hasSpots_c_min))
                arrayfun(@(array1,array2) phase.setNofConflicts(array1, array2), ...
                         idconflict_min(hasSpots_c_min), totspots_min(hasSpots_c_min));

                arrayfun(@(array) phase.setHasConflicts(array, true), idconflict_min(hasSpots_c_min));
                arrayfun(@(array) phase.setHasConflicts(array, false), idconflict_min(~hasSpots_c_min));
            end % to update conflicts and summary

            idspots_c_min = [idspots_c_min{:}];
            redo_rec = true;
            redo_assemble = true;
        else
            idspots_c_min = [];
            idconflict_min = [];
            redo_rec = false;
            redo_assemble = false;

        end % end if ~isempty(idconflict_min)

        idspots_min = [idspots_min{:}];
        redo_rec = true;
        redo_assemble = true;
    else
        idconflict_min = [];
        idspots_c_min  = [];
        idspots_min    = [];
        redo_rec = false;
        redo_assemble = false;
    end % end if ~isempty(grainIDs_min)

    % get remaining conflicts
    conflicts = phase.getAllValuesFromSummaryField('idconflict',false);
    if ~isempty(conflicts)
        [conflicts, indexes_c] = gtGetSinglesValues(conflicts, []);
        conflicts_list   = [indexes_c, conflicts];
        conflicts_unique = unique(sort(conflicts_list,2),'rows');
        conflicts        = unique(conflicts_list);
    else
        conflicts        = [];
        conflicts_list   = [];
        conflicts_unique = [];
    end

    % saving information
    out{phaseid}.deselected_grains = grainIDs_desel';
    out{phaseid}.modified_grains   = unique([grainIDs_min, idconflict_min])';
    out{phaseid}.modified_difspots = unique([idspots_min, idspots_c_min]);
    out{phaseid}.conflicts_list    = conflicts_list;
    out{phaseid}.conflicts_unique  = conflicts_unique;
    out{phaseid}.conflicts         = conflicts;
    out{phaseid}.spot2grain        = {spot2grain, spot2phase};

    % update sample structure for current phase
    sample.phases{phaseid} = phase;

    if redo_rec && redo_assemble && ~isempty(out{phaseid}.modified_grains)
        disp(['  For phase ' num2str(phaseid) ' is recommended to run again the Reconstruction for those grains: '])
        disp(out{phaseid}.modified_grains')
        disp('  and assemble this phase.')
        check = inputwdefault('Do you want to run gtReconstructGrain and gtChangeThreshold3D for those grains? [y/n]','y');
        if strcmpi(check,'y')
            disp('It may take sometimes... Be patient :-)')
            diary off
            for jj = 1 : length(out{phaseid}.modified_grains)
                gtReconstructGrains(out{phaseid}.modified_grains(jj), out{phaseid}.modified_grains(jj), pwd(), phaseid);
                gtChangeThreshold3D(out{phaseid}.modified_grains(jj), out{phaseid}.modified_grains(jj), pwd(), phaseid);
            end
            diary on
            disp('  gtReconstructGrain and gtChangeThreshold3D have been executed for grains:')
            disp(out{phaseid}.modified_grains')
        end
    elseif redo_assemble
        disp(['  For phase ' num2str(phaseid) ' is recommended to run again the Assemble.'])
    end

    if ~save_flag
        disp('Remember that spot2grain.mat has not been updated after the analysis.')
        disp('  You should do that afterward to be consistent:')
        disp('  The updated spot2grainmat is stored in')
        disp('    4_grains/out.mat:out{phaseid}.spot2grain')
        disp('  The updated sample.mat is the output from this function: don''t loose it!')
    end

end % end for phaseid

if exist(out_root,'file')
    copyfile(out_root, outFile);
end
save(out_root,'out', '-v7.3');
disp(['Saved output in ' out_root])

if save_flag
    copyfile(sample_root, sampleFile);
    sample.saveToFile(sample_root);
    disp(['Saved ' sample_root])
end


diary off

end % end of function


%%
% sub-functions
function output = sfUpdateSpot2Grain(grains_list, difspots_list, spot2grain, spot2phase)

% update spot2grain and spot2phase
for jj = 1 : length(difspots_list)
    for ii = 1 : length(grains_list)
        index = find(spot2grain{difspots_list(jj)}==grains_list(ii));
        if ~isempty(index)
            spot2grain{difspots_list(jj)}(index)=[];
            spot2phase{difspots_list(jj)}(index)=[];
            disp(['  Removed conflict with difspotID: ' num2str(difspots_list(jj)) ' for grainID : ' num2str(grains_list(ii))])
        end
    end
end
disp(['Found ' num2str(length(difspots_list)) ' difspots in conflict with ' num2str(length(grains_list)) ' grains'])
output.spot2grain = spot2grain;
output.spot2phase = spot2phase;

end % end of sfUpdateSpot2Grain

