function data = gtHexagonalUnitCell(varargin)
% GTHEXAGONALUNITCELL  Draws the hexagonal unit cell with crystallographic axes
%                      for Euler angles (0,0,0)
%
%     data = gtHexagonalUnitCell(varargin)
%     ------------------------------------
%     Draws the unit cell using this convention:
%     Xs // to <2-1-10> or <100> axis (a-axis)
%     Ys // to 
%
%     Crystallographic axes are Xc, Yc, Zc which corresponds to a,b,c.
%     For hexagonal unit cells, there are the equivalents axes a1,a2,a3,c in
%     which a1//a, a2 is a1 rotated 120 deg counterclockwise, a3 is a2 rotated
%     120 degrees counterclockwise.
%     Sample axes are Xs,Ys,Zs. Please refer to the
%     DCTdoc_Crystal_orientation_descriptions.pdf in the DCT_Documentation
%     folder for the graintracking code.
%


app.centered     = false;
app.draw         = true;
app.ratio        = 1.5857;
app.ind          = 0;
app.vertices     = false;
app.numbers      = false;
app.patch        = false;
app.sampleaxes   = false;
app.crystaxes    = true;
app.view         = [103 20];
app.add_vertices = false;

app = parse_pv_pairs(app, varargin);

% lattice axes' length; c is normalized to 1
c=1;
a=1/app.ratio;

% calculation of vertices
% first 6 are for 000-1 plane
% first is at X=0 and then counterclockwise
v=0:60:360;
x0 = a*cosd(v);
y0 = a*sind(v);
if app.centered
    z0 = repmat(-c/2,1,length(v));
    z1 = repmat(+c/2,1,length(v));
else
    z0 = zeros(1,length(v));
    z1 = repmat(c,1,length(v));
end
vertices = [[x0;y0;z0],[x0;y0;z1]]';
vertices(7,:)=[];
vertices(end,:)=[];

if app.add_vertices
    vertices(end+1,:) = [0 0 z0(1)];
    vertices(end+1,:) = [0 0 z1(1)];

    vertices(end+1,:) = [0 0 (z1(1)-z0(1))/3];
    vertices(end+1,:) = [0 0 (z1(1)-z0(1))/3*2];

    vertices(end+1,:) = [0 0 (z1(1)-z0(1))/4];
    vertices(end+1,:) = [0 0 (z1(1)-z0(1))/4*2];
    vertices(end+1,:) = [0 0 (z1(1)-z0(1))/4*3];
end

% hexagon face directions
% vertices for patch
% hkil of plane normals
faces_sides = [1 2 8 7; 4 5 11 10; 5 6 12 11; 2 3 9 8; 3 4 10 9; 6 1 7 12];
faces_end   = [1 2 3 4 5 6; 7 8 9 10 11 12]; %
hkil        = [1 0 -1 0; -1 0 1 0; 0 -1 1 0; 0 1 -1 0; -1 1 0 0; 1 -1 0 0; ...
               0 0 0 -1; 0 0 0 1]'; % column vectors

% for crystallographic axes and labels
orig=repmat([0 0 z0(1)],4,1);
xcrys=[1;-0.5;-0.5;0];
ycrys=[0;0.866;-0.866;0];
zcrys=[z0(1);z0(1);z0(1);z1(1)+0.4];
lcrys={'a1 [2-1-10]';...
       'a2 [-12-10]';...
       'a3 [-1-120]';...
       'c [0001]'};

if app.draw
    figure;
    % basal planes
    line(x0,y0,z0);
    line(x0,y0,z1);
    hold on;
    % prismatic planes
    for ii=1:length(x0)
        plot3([x0(ii) x0(ii)],[y0(ii) y0(ii)],[z0(ii) z1(ii)], 'b-', 'LineWidth', 1)
    end
    % cryst axes
    if app.crystaxes
        quiver3(orig(:,1),orig(:,2),orig(:,3),xcrys,ycrys,zcrys,0,'k')
        text(xcrys-0.1,ycrys+0.1,zcrys,lcrys,'Color',[0 0 0])
    end
    view(app.view)
    if app.sampleaxes
        % sample reference system : Xs, Ys, Zs vectors
        quiver3(-1.5,0,0,3,0,0,0,'r');
        quiver3(0,-1.5,0,0,3,0,0,'g');
        quiver3(0,0,z0(1)-0.5,0,0,(z1(1)-z0(1))+1.5,0,'b');
        text(1.5+0.1,0,0,'Xs','Color',[1 0 0])
        text(0,1.5+0.1,0,'Ys','Color',[0 1 0])
        text(0,0,z1(1)+1.1,'Zs','Color',[0 0 1])
    end
    if app.patch
        if app.ind >=1 && app.ind <=6
          patch('Vertices', vertices, 'Faces', faces_sides(app.ind,:), ...
            'FaceVertexCData', repmat([0 0.8 0],length(app.ind),1), 'FaceColor', 'flat');
        elseif app.ind==7 || app.ind==8
          patch('Vertices', vertices, 'Faces', faces_end(app.ind-6,:), ...
            'FaceVertexCData', repmat([0 0.8 0],length(app.ind-6),1), 'FaceColor', 'flat');
        elseif app.ind==0
          patch('Vertices', vertices, 'Faces', faces_sides, ...
            'FaceVertexCData', repmat([0 0.8 0],6,1), 'FaceColor', 'flat');
          patch('Vertices', vertices, 'Faces', faces_end, ...
            'FaceVertexCData', repmat([0 0.8 0],2,1), 'FaceColor', 'flat');
        end
    end
    axis equal
    set(gca,'Visible','off')

    if app.numbers
        text(vertices(:,1),vertices(:,2),vertices(:,3)+0.1,num2str([1:length(vertices)]'),'Color',[0 0 0])
    end
    if app.vertices
        plot3(vertices(:,1),vertices(:,2),vertices(:,3),'.k')
    end
end % end draw

if nargout == 1
    data.vertices    = vertices;
    data.faces_sides = faces_sides;
    data.faces_end   = faces_end;
    data.hkil        = hkil;
    data.orig        = orig;
    data.xcrys       = xcrys;
    data.ycrys       = ycrys;
    data.zcrys       = zcrys;
    data.lcrys       = lcrys;
    if app.draw
        data.hf      = gcf;
    end
end

end % end of function
