function [cmap, Vsst, Vc_symm] = gtIPFCmap(phaseid, LD, varargin)
% GTIPFCMAP  Returns a colour map for the specified phase and lab/sample direction.
%
%     [cmap, Vsst, Vc_symm] = gtIPFCmap(phaseid, LD, varargin)
%     --------------------------------------------------------
%     Makes an IPF colourmap of the lab_ direction (LD) in the crystal 
%     coordinates. This can be specified as a second argument, otherwise we
%     assume the z-axis [0 0 1]
%
%     INPUT:
%       phaseid          = <int>     ID of the specific phase {1}
%       LD               = <double>  Lab direction to study   {[0 0 1]}
%
%     OPTIONAL INPUT (varargin as a list of pairs, see parse_pv_pairs.m)
%       'saturate'       = <logical> Saturate colors in the SST         {true}
%       'save'           = <logical> Save color map to cmap_phase<phaseid>.mat
%                                                                       {true}
%       'crystal_system' = <string>  Legacy to set (force) the crystal system
%                                    for old datasets
%   
%     OUTPUT:
%       cmap             = <double>  RGB color map, color code is the inverse
%                                    pole figure in the SST
%       Vsst             = <double>  Row vectors expressed in crystal frame
%                                    moved to the SST triangle
%       Vc_symm          = <double>  Row vectors expressed in crystal frame
%
%
%     Version 003 08-01-2013 by LNervo
%       Added second output argument: Vsst
%
%     Version 002 03-01-2013 by YGuilhem
%       Renamed gtIVPCmap to gtIPFCmap, changed colour saturation
%
%     Version 001 15-11-2012 by YGuilhem
%       Built by gathering old gtIVPcubCmap and gtIVPhexCmap functions

% Default parameters
par.saturate = true;
par.save = false;
par.crystal_system = '';
par = parse_pv_pairs(par, varargin);

% Set default phaseid
if ~exist('phaseid', 'var') || isempty(phaseid)
    phaseid = 1;
end

% Set default lab direction if LD empty
if ~exist('LD', 'var') || isempty(LD)
    LD = [0 0 1];
end

% Get the r-vectors
r_vectors = [];
if exist(fullfile('4_grains', sprintf('phase_%02d', phaseid), 'r_vectors.mat'), 'file')
    load(fullfile('4_grains', sprintf('phase_%02d', phaseid), 'r_vectors.mat'));
    if size(r_vectors,2) == 4
        r_vectors = r_vectors(:, 2:4);
    end
elseif exist(fullfile('4_grains', sprintf('phase_%02d', phaseid), 'index.mat'), 'file')
    grain = [];
    load(fullfile('4_grains', sprintf('phase_%02d', phaseid), 'index.mat'), 'grain');
    r_vectors = gtIndexAllGrainValues(grain, 'R_vector', [], 1:3, []);
else
    gtError('gtIPFCmap:missing_file', 'Could not find r_vectors.mat or index.mat file!');
end

% Get the symmetry operators
if isempty(par.crystal_system)
    parameters = load('parameters.mat');
    par.crystal_system = parameters.parameters.cryst(phaseid).crystal_system;
end
tmp = gtCrystGetSymmetryOperators(par.crystal_system);
if isfield(tmp, 'g3')
    symm = {tmp.g3};
else
    symm = {tmp.g};
end

% Get sample to crystal rotation matrices
%   Gcry = g Gsam
%   g = Rod2g(R_vector)' = U = gtMathsRod2RotMat(R_vector)
sam2crys = gtMathsRod2RotMat(r_vectors);

% Compute LDc = LD expressed in the crystal CS
LDc = gtVectorLab2Cryst(LD, sam2crys);

% Get the color and vector in the SST triangle
[map, Vsst, ~, Vc_symm] = gtCrystVector2SST(LDc, par.crystal_system, symm, par.saturate);

cmap = [ 0 0 0 ; map ];

if par.save
    fileName = fullfile('6_rendering', sprintf('cmap_phase%02d.mat', phaseid));
    save(fileName, 'cmap');
end

end % end of function
