function [hf, hp, gids] = gtRenderGrains(grains, varargin)
% GTRENDERGRAINS plots a 3D rendered image from a grain map.
%
%     [hf, hp, gids] = gtRenderGrains(grains, varargin)
%     -----------------------------------------------------------
%
%     Attention! It can take very long if the number of grains is large.
%     Note that displayed volume is shifted with one pixel in Z.
%
%     If more than one connected grains are rendered, the fastest way is to 
%     do it on a dilated volume (the more space filling, the faster) with the
%     default patchmode='free_interp' option.
%     
%     After rendering you can use the mouse with the "Rotate 3D' tool or 
%     'view' command to change the view (e.g. view(20,60)), and the 
%     'camlight' command to add illumination (e.g. camlight(-45,-45)), if
%     some grains are poorly visible.
%
%     For more information on rendering, see Lighting Overview in the
%     Matlab help. 
%
%     
%     INPUT:
%       grains = labelled grain map (e.g. as in volume_dilated.mat)
%
%     OPTIONAL INPUT - varargin:
%       grainids  = grain ID-s to be shown; if empty, all are considered
%       zmark     = marks a given Z layer {empty}
%       xkeep     = X region to keep; grains that touch this region will 
%                   be displayed {1:size(grains,2)}
%       ykeep     = Y region to keep {1:size(grains,1)}
%       zkeep     = Z region to keep {1:size(grains,3)}
%       xcut      = X region to cut out; grains that touch this region will
%                   be excluded completely (e.g. xcut=1:100); {empty}
%       ycut      = Y region to cut out {empty}
%       zcut      = Z region to cut out {empty}
%       patchmode = patch mode:
%                     'all'          - the surface of each grain will be rendered; 
%                     'free_flat'    - only visible free surfaces will be 
%                                      rendered (faster); flat face colouring;
%                                      grain boundaries might look worse
%                     {'free_interp'}- only visible free surfaces will be
%                                      rendered; RGB colours are
%                                      interpolated on the grain boundaries
%       cmap      = colormap (nx3); if empty, gtRandCmap is used; cmap(1,:)
%                   is used for grainID=1, etc.
%       showaxes  = show axes in figure {true}
%       showlabel = show axes labels {true}
%       grid      = show grid in figure {true}
%       viewaz    = azimuth angle of view {30}
%       viewel    = elevation angle of view {40}
%       lighting  = light rendering method ({'phong'}/'gouraud'/'flat'/'none')
%       lightaz   = light direction azimuth angle {45}
%       lightel   = light direction elevation angle {45}
%       ambient   = strength of ambient light {0.5}
%       diffuse   = strength of diffuse reflected light {0.5}
%       specular  = strength of specular (directional) reflected light {0.3}
%
%
%     OUTPUT:
%       hf     = handle of figure
%       hp     = handles of all plotted isosurface patches
%       gids   = final grain ID-s displayed
%
%   Version 002 19-09-2012 by PReischig
%     Cleaned and pimped.
% 
%   Version 001 by gtTeam
%
%
%
%


par.grainids  = [];
par.patchmode = 'free_interp';
par.xcut      = [];
par.ycut      = [];
par.zcut      = [];
par.xkeep     = 1:size(grains,2);  % volume is transposed
par.ykeep     = 1:size(grains,1);  % volume is transposed
par.zkeep     = 1:size(grains,3);
par.cmap      = [];
par.zmark     = [];
par.showaxes  = true;
par.showlabel = true;
par.grid      = true;
par.reverse_x = false;
par.reverse_y = false;
par.reverse_z = false;
par.viewaz    = 30;
par.viewel    = 40; 
par.lighting  = 'phong';
par.lightaz   = 45;
par.lightel   = 45;
par.ambient   = 0.5;
par.diffuse   = 0.5;
par.specular  = 0.3;

try
par = parse_pv_pairs(par, varargin);
catch mexc
    gtPrintException(mexc);
    help(mfilename);
    return
end

tic();

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Grain ID-s to be rendered
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Transpose X-Y for true orientation
grains = permute(grains, [2 1 3]);

% Grains ID-s to be shown
grainids = unique(grains(par.xkeep, par.ykeep, par.zkeep));
if ~isempty(par.grainids)
    grainids = intersect(grainids, par.grainids);
end

% Grains ID-s to be cut
cutids   = unique( grains(par.xcut, par.ycut, par.zcut) );
grainids = setdiff(grainids, cutids);

% Remove background and conflicting grains
grainids = setdiff(grainids, [-1 0]);
gids     = grainids; 

if isempty(grainids)
    disp('No grains to display.')
    return
end

% Pad grain map to avoid missing patches on the edge
grains = padarray(grains, [1 1 1], 0);

%%%%%%%%%%%%%%%%%%%%%%
%% Create patches
%%%%%%%%%%%%%%%%%%%%%%
hf = figure();
ha = axes('parent', hf);

if isempty(par.cmap)
    par.cmap = gtRandCmap(max(grainids)+1);
    par.cmap(1,:) = [];
end

set(hf, 'Colormap', par.cmap);

switch par.patchmode
    
    case 'all'
        % Loop through all active grains and create patch
        for ii = 1:length(grainids)
            fprintf('rendering grain %d of %d...\n', ii, length(grainids))
            
            vol_bin    = (grains == grainids(ii));
            vol_smooth = smooth3(vol_bin, 'box', 3);
            
            hp(ii) = patch(isosurface(vol_smooth, 0.5), 'EdgeColor', 'none', ...
                     'FaceColor', par.cmap(grainids(ii), :), 'Parent',ha); 
        end

    case 'free_flat'
        % Patches are created from free surface, then coloured according 
        % to nearest grain ID.

        vol_bin = false(size(grains));
        for ii = 1:length(grainids)
            vol_bin(grains==grainids(ii)) = true;
        end
        
        vol_smooth = smooth3(vol_bin, 'box', 3);
        hp         = patch(isosurface(vol_smooth, 0.5), 'EdgeColor', 'none',...
                     'Parent',ha);
        verts      = get(hp,'vertices');
   
        % Dilation of active grain ID-s is needed for later interpolation;
        % dilation changes original voxels, thus they need to be reset.
        gr_merged          = zeros(size(grains), 'single');
        gr_merged(vol_bin) = grains(vol_bin);
        gr_merged          = imdilate(gr_merged, ones(3,3,3));
        gr_merged(vol_bin) = grains(vol_bin);
        
        % Get vertex colour data from interpolation
        cdata = interp3(gr_merged, verts(:,1), verts(:,2), verts(:,3), 'nearest');
        
        set(hp, 'FaceColor', 'flat', 'FaceVertexCData', cdata, 'CDataMapping', 'direct');

    case 'free_interp'
        % Patches are created from free surface. For improved grain boundary
        % rendering, RGB interpolation is used.

        vol_bin = (grains == grainids(1));
        for ii = 2:length(grainids)
            vol_bin = vol_bin | (grains == grainids(ii));
        end

        vol_smooth = smooth3(vol_bin, 'box', 3);
        isoSurf    = isosurface(vol_smooth, 0.5);
        hp         = patch(isoSurf, 'EdgeColor', 'none', 'Parent', ha);
        verts      = get(hp, 'vertices');

        % Dilation needed before interpolation.
        grainsRegion = grains .* int16(vol_bin);
        dilatedGrains = imdilate(grainsRegion, ones(3, 3, 3));
        dilatedGrains(vol_bin) = grainsRegion(vol_bin);
        vol_bin = logical(dilatedGrains);

        gr_R = zeros(size(grains), 'single');
        gr_G = gr_R;
        gr_B = gr_R;

        gr_R(vol_bin) = par.cmap(dilatedGrains(vol_bin), 1);
        gr_G(vol_bin) = par.cmap(dilatedGrains(vol_bin), 2);
        gr_B(vol_bin) = par.cmap(dilatedGrains(vol_bin), 3);

        % Colour data of the vetrices
        cdata_R = interp3(gr_R, verts(:,1), verts(:,2), verts(:,3), 'linear');
        cdata_G = interp3(gr_G, verts(:,1), verts(:,2), verts(:,3), 'linear');
        cdata_B = interp3(gr_B, verts(:,1), verts(:,2), verts(:,3), 'linear');

        set(hp, 'FaceColor','interp', 'FaceVertexCData', [cdata_R, cdata_G, cdata_B]);
end

set(hp, 'UserData', grainids);

%%%%%%%%%%%%%%%%%%%%%%
%% Set light and axes
%%%%%%%%%%%%%%%%%%%%%%
set(hp, 'AmbientStrength',  par.ambient)
set(hp, 'DiffuseStrength',  par.diffuse)
set(hp, 'SpecularStrength', par.specular)

view(par.viewaz, par.viewel)

% Set axes according to gtGrainsManager
if par.reverse_z
    set(ha,'Zdir','reverse')
end
if par.reverse_y
    set(ha,'Ydir','reverse')
end
if par.reverse_x
    set(ha,'Xdir','reverse')
end

axis(ha, 'equal')
axis(ha, 'vis3d')

camlight(par.lightaz, par.lightel);
lighting(par.lighting)


if ~par.showaxes
    set(ha, 'xTick', []);
    set(ha, 'yTick', []);
    set(ha, 'zTick', []);
end

if par.showlabel
    xlabel(ha, 'X')
    ylabel(ha, 'Y')
    zlabel(ha, 'Z')
end

if par.grid
    grid(ha, 'on')
else
    grid(ha, 'off')
end

% Draw rectangle at a specified layer
if ~isempty(par.zmark)
    ll = par.zmark + 1;  % original volume was padded
    sx = xlim;
    sy = ylim; 
    line([sx(1) sx(1) sx(2) sx(2) sx(1)]',[sy(1) sy(2) sy(2) sy(1) sy(1)]',...
        [ll ll ll ll ll]', 'Color','r','LineWidth',1);
end

hr = rotate3d(hf);
set(hr, 'RotateStyle', 'box', 'Enable', 'on');

drawnow

toc()

end
