/* jacobs_ray3d
 Calculates intersection lengths of a line with a voxel grid
 Implementation of algorithm described by F. Jacobs et.al.
 
 F. Jacobs, E. Sundermann, B. De Sutter, M. Christiaens and I. Lemahieu, ["A Fast
 Algorithm to Calculate the Exact Radiological Path through a Pixel or Voxel
 Space"
 Journal of Computing and Information Technology - CIT 6 1998, 1, pp 89-94]
 
 Written by David Szotten 2005-2006
 D.Szotten@student.manchester.ac.uk
 
 Copyright (c) The University of Manchester 2006 */


#include <mex.h>
#include <math.h>
#include <matrix.h>
#include <stdlib.h>


void jacobs_ray_3d(int im_size, double *start, double *end, int *ray_index, double *ray_data, int *n_entries, 
		  double voxel_size);




#define MINX(x,y) ((x) < (y) ? (x) : (y))
#define MAXX(x,y) ((x) > (y) ? (x) : (y))

#define PRECISION 0.00000001

void sort_and_set(int *tmp_arr_index, double *tmp_arr_data, int n_entries, double *pr ,mwIndex *ir, mwIndex *jc);
void qsort_indexed_list(int *index, double *values, int l, int r);
void swop_dbl(double *a, double *b);
void swop_int(int *a, int *b);

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
	/*check number of arguments */
	if(nrhs != 8) {
		mexWarnMsgTxt("Eight inputs required. Aborting");
		return;
		}
	else if(nlhs > 1) {
		mexWarnMsgTxt("Too many output arguments. Aborting");
		return;
	}

	/*check the arguments */
	if(!(mxIsDouble(prhs[0]) && mxGetScalar(prhs[0]) > 0)) {
		mexWarnMsgTxt("Side length must be a positive integer");
		return;
		}


	int im_size;
	double p1_x, p1_y, p1_z, p2_x, p2_y, p2_z, voxel_size;
	double start[3], end[3];





	/*read variables */
	im_size=(int) mxGetScalar(prhs[0]);
	start[0]=p1_x=mxGetScalar(prhs[1]);
	start[1]=p1_y=mxGetScalar(prhs[2]);
	start[2]=p1_z=mxGetScalar(prhs[3]);
	end[0]=p2_x=mxGetScalar(prhs[4]);
	end[1]=p2_y=mxGetScalar(prhs[5]);
	end[2]=p2_z=mxGetScalar(prhs[6]);
	voxel_size = mxGetScalar(prhs[7]);

	/*allocate memory for the matrix. 3*im_size entries should be enough */
	/* storing as column vector uses less space (due to implementation of sparse) */
/*	plhs[0] = mxCreateSparse(im_size*im_size*im_size, 1, 3*im_size ,mxREAL); */
  plhs[0] = mxCreateSparse((mwSize)(im_size*im_size*im_size),1,3*im_size,mxREAL);

	double *output_matrix_pr;
	mwIndex *output_matrix_ir, *output_matrix_jc;

	output_matrix_pr = mxGetPr(plhs[0]);
    	output_matrix_ir = mxGetIr(plhs[0]);
    	output_matrix_jc = mxGetJc(plhs[0]);


	/* to store data before sorting (for sparse matrix) */
	int *tmp_arr_index;
	double *tmp_arr_data;
	int tmp_counter;

	tmp_arr_index=mxCalloc(im_size*im_size*im_size,sizeof(int));
	tmp_arr_data=mxCalloc(im_size*im_size*im_size,sizeof(double));

	if (tmp_arr_index == NULL || tmp_arr_data == NULL ) {
		mexWarnMsgTxt("Not enough memory!");
		return;
	}


	jacobs_ray_3d(im_size, start, end, tmp_arr_index, tmp_arr_data, &tmp_counter, voxel_size);
	/*coor_to_voxel(im_size, tmp_arr_index_i, tmp_arr_index_j, tmp_arr_index, &tmp_counter);*/
	
	/*int tmp;
	for(tmp=0;tmp<tmp_counter;tmp++)
		mexPrintf("%d: %f\n", tmp_arr_index[tmp], tmp_arr_data[tmp]);*/

	sort_and_set(tmp_arr_index, tmp_arr_data, tmp_counter, output_matrix_pr, output_matrix_ir, output_matrix_jc);
}


void sort_and_set(int *tmp_arr_index, double *tmp_arr_data, int n_entries, double *pr ,mwIndex *ir, mwIndex *jc)
{
	 /* if n_entries == 0, store empty matrix */
	 /*mexPrintf("sorted, entries: %d\n", n_entries);/**/

	if(n_entries != 0)
		qsort_indexed_list(tmp_arr_index, tmp_arr_data, 1, n_entries);

	int n, pos;

	jc[0]=0;
	jc[1]=n_entries-1*0;
	pos=0;
	for(n=0; n<n_entries-1*0; n++) {
		if(tmp_arr_data[n] != 0) {
			ir[pos]=tmp_arr_index[n];
			pr[pos++]=tmp_arr_data[n];
			/*mexPrintf("n: %d, ir: %d, pr: %f\n", n, ir[n], pr[n]);/**/
		}
		else
			jc[1]--;
	}

/*	int m;
	int o=0;
	for(n=0; n<36; n++) {
		jc[n]=o;
		for(m=0; m<n_entries-1;m++)
			if(n==tmp_arr_index[m]) {
				ir[o]=0*n;
				pr[o++]=tmp_arr_data[m];
				mexPrintf("n: %d, m: %d, o: %d, ir: %d, pr: %f\n", n, m, o, ir[o-1], pr[o-1]);
				}
		}
	jc[36]=o;
*/

}


void qsort_indexed_list(int *index, double *values, int l, int r)
/* to sort the entries in index ascending, and entries in values to match
[1 3 2                  [1 2 3
   a b c]    becomes     a c b] */
{
    /*mexPrintf("qsort, left: %d, right: %d \n", l, r);*/

    int l1, r1;

    if (l < r) {
	l1=l;
	r1=r;

	do {
	    while (l1 < r && index[l1-1] <= index[l-1])
		l1++;
	    while(l < r1 && index[r1-1] >= index[l-1])
		r1--;
	    if (l1 < r1) {
		swop_int( &index[l1-1], &index[r1-1]);
		swop_dbl( &values[l1-1], &values[r1-1]);
	    }
	} while (l1 < r1);

	swop_int( &index[l-1], &index[r1-1]);
	swop_dbl( &values[l-1], &values[r1-1]);

	qsort_indexed_list(index, values, l, r1-1);
	qsort_indexed_list(index, values, r1+1, r);
    }
}


void swop_int(int *a, int *b)
/* swops the values of a and b */
{
    double t;
    t=*a;
    *a=*b;
    *b=t;
}

void swop_dbl(double *a, double *b)
/* swops the values of a and b */
{
    double t;
    t=*a;
    *a=*b;
    *b=t;
}

