function proj_geom = gtGeoProjForReconstruction(projvec, omega, graincenter,...
                     bbpos, bboff, labgeo, samgeo, recgeo, rectype)
% GTGEOPROJFORRECONSTRUCTION Projection coordinates required for
%                            reconstructions.
%
% proj_geom = gtGeoProjForReconstruction(projvec, omega, ...
%             graincenter, bbpos, bboff, labgeo, samgeo, recgeo)
% 
% ----------------------------------------------------------------
%
% Reconstruction geometries
%   'ASTRA_grain' - for a single grain placed in the origin of the 
%                   reconctructed ROI (requires 'graincenter')
%   'ASTRA_full'  - for a reconstruction of the complete sample volume
%                   (does not use 'graincenter')
% 
%
%     INPUT:
%       projvec     = projection vectors (beam direction) in the 
%                     SAMPLE reference (nx3)
%       omega       = omega rotational positions corresponding to 
%                     the projection vectors (in degrees) (nx1)
%       graincenter = grain center in the SAMPLE reference (1x3)
%       bbpos       = bounding box positions on the detector (in pixels)
%                     [corner_U corner_V width height] (nx4)
%       bboff       = additional bounding box U,V offsets relative to
%                     position 'bbpos' (n,2)
%       labgeo      = LABoratory reference as in parameters.labgeo
%       samgeo      = SAMPLE reference as in parameters.samgeo
%       recgeo      = REConstruction reference as in parameters.rec
%       rectype     = <char> reconstruction geometry:
%                     'ASTRA_grain' or 'ASTRA_full'
%
%     OUTPUT:
%       proj_geom   = input for the specified reconstruction geometry:
%                       'ASTRA_grain' and 'ASTRA_full':
%                         Parameters defined relative to the sample (nx12):
%                             [projection vector, detector position, ...
%                              detector vector U, detector vector V];
%                       
%

if isempty(bboff)
    bboff = zeros(size(bbpos,1),2);
end


% Bounding box center U,V coordinates
bbcent_det  = [ bbpos(:,1) + (bbpos(:,3)-1)/2 + bboff(:,1) , ...
                bbpos(:,2) + (bbpos(:,4)-1)/2 + bboff(:,2) ];

% Detector origin
labgeo.detorig = gtGeoDetOrig(labgeo);

% Bounding box center X,Y,Z coordinates in LAB and REC reference
bbcent_lab  = gtGeoDet2Lab(bbcent_det, labgeo, 0);
bbcent_rec  = gtGeoLab2Sam(bbcent_lab, omega, labgeo, recgeo, 0);

% Detector orientation vectors in REC reference
detdiru = labgeo.detdiru./sqrt(sum(labgeo.detdiru.^2))*labgeo.pixelsizeu;
detdirv = labgeo.detdirv./sqrt(sum(labgeo.detdirv.^2))*labgeo.pixelsizev;

detdiru_rec = gtGeoLab2Sam(detdiru(ones(length(omega),1),:), omega, labgeo, recgeo, 1);
detdirv_rec = gtGeoLab2Sam(detdirv(ones(length(omega),1),:), omega, labgeo, recgeo, 1);

% Normalised projection vector in REC reference
projvec_rec      = gtGeoSam2Sam(projvec, samgeo, recgeo, 1);
projvec_rec_norm = sqrt(sum(projvec_rec.^2,2));
projvec_rec      = projvec_rec./projvec_rec_norm(:,[1 1 1]);

% Grain center in REC reference
graincenter_rec = gtGeoSam2Sam(graincenter, samgeo, recgeo, 0);


switch (rectype)
    case {'ASTRA_grain'}
        bbcent_rec = bbcent_rec - graincenter_rec(ones(size(bbcent_rec,1),1),:);
        
        proj_geom = [projvec_rec, bbcent_rec, detdiru_rec, detdirv_rec];

    case {'ASTRA_full'}
        proj_geom = [projvec_rec, bbcent_rec, detdiru_rec, detdirv_rec];
end   


end % of function


