function g = gtMathsEuler2RotMat(euler)
% GTMATHSEULER2ROTMAT  Convert orientation matrices to Euler angles (degrees).
%                      These follow Bunge convention (ZXZ) [phi1, phi, phi2].
%
%     g = gtMathsEuler2RotMat(euler)
%     -------------------------------------------------------------------------
%
%     INPUT:
%       euler = <double> M Euler angles triplets, as row vectors (Mx3 matrix).
%
%     OUTPUT:
%       g     = <cell>   Cell containing M orientation matrices (3x3 matrices).
%                        See Randle & Engler, 2000 for more details.
%
%     Version 002 17-10-2012 by YGuilhem
%       Vectorize function to process quickly lots of Euler triplets
%
%     Version 001 15-06-2012 by YGuilhem

% Check size of row vectors
if size(euler, 2) ~= 3 
    gtError('Euler:wrong_vector_size', ...
        'Given matrix should be M x 3 (a list of 3 components row vectors)!');
    throw(Mexc);
end

% Get number of Euler triplets
M = size(euler, 1);

% Reshape angles components 
phi1 = reshape(euler(:, 1), [1 1 M]);
phi  = reshape(euler(:, 2), [1 1 M]);
phi2 = reshape(euler(:, 3), [1 1 M]);

% Compute intermediate values 
c1 = cosd(phi1);
cg = cosd(phi);
c2 = cosd(phi2);
s1 = sind(phi1);
sg = sind(phi);
s2 = sind(phi2);

% Compute g matrices components
g_vec = [  c1.*c2 - s1.*cg.*s2 ,  s1.*c2 + c1.*cg.*s2 ,  sg.*s2 ; ...
          -c1.*s2 - s1.*cg.*c2 ,  c1.*cg.*c2 - s1.*s2 ,  sg.*c2 ; ...
           s1.*sg              , -c1.*sg              ,  cg     ];

% Allocate & fill g
g = cell(1, M);
for ig=1:M
    g{ig} = g_vec(:, :, ig);
end
% Convert to cell (slower than for loop, strange...)
%g = num2cell(g_vec, [1 2]);

end % end of function
