function g = gtMathsRod2RotMat(r)
% GTMATHSROD2ROTMAT  Convert Rodrigues vectors to orientation matrices.
%
%     g = gtMathsRod2RotMat(r)
%     -------------------------------------------------------------------------
%
%     INPUT:
%       r = <double>  M Rodrigues vectors, as row vectors (Mx3 matrix).
%
%     OUTPUT:
%       g = <cell>    Cell containing M orientation matrices (3x3 matrices).
%                     See Randle & Engler, 2000 for more details.
%
%     Version 001 15-06-2012 by YGuilhem
%       Vectorize function to process quickly a lot of Rodrigues vectors
%
%     Version 001 15-06-2012 by YGuilhem

% Check size of r row vectors
if size(r, 2) ~= 3
    gtError('Rodrigues:wrong_vector_size', ... 
        'Given matrix size is not Mx3!');
end

% Get number of r vectors
M = size(r, 1);

% Set epsilon to avoid singular values
epsilon = 1.e-9;

% Reshape r with a transpose
r = reshape(r.', [1 3 M]);

% Compute the norm of each r vector and normalize r -> n
r_norm = sqrt(max(sum(r.^2, 2), epsilon));
n = r./r_norm(:, [1 1 1], :);

% Separate each component into vectors
n0 = zeros(1, 1, M);
n1 = n(:, 1, :);
n2 = n(:, 2, :);
n3 = n(:, 3, :);

% Theta angle
theta = 2 * atan(r_norm);
costheta = cos(theta);
costheta = costheta([1 1 1], [1 1 1], :);
sintheta = sin(theta);
sintheta = sintheta([1 1 1], [1 1 1], :);

I = eye(3);
I = I(:, :, ones(1, M));

w = [ n0 ,  n3 , -n2 ; ...
     -n3 ,  n0 ,  n1 ; ...
      n2 , -n1 ,  n0 ];

% Squared w matrix
w2 = zeros(size(w));
for ig=1:M
    w2(:, :, ig) = w(:, :, ig)^2;
end

% Compute g_vec matrices
g_vec = I + sintheta.*w + (1-costheta).*w2;

% Allocate & fill g
g = cell(1, M);
for ig=1:M
    g{ig} = g_vec(:, :, ig);
end
% Convert to cell (slower than for loop, strange...)
%g = num2cell(g_vec, [1 2]);

end % end of function
