function euler = gtMathsRotMat2Euler(g)
% GTMATHSROTMAT2EULER  Convert orientation matrices to Euler angles (degrees).
%                      These follow Bunge convention (ZXZ) [phi1, phi, phi2].
%
%     euler = gtMathsRotMat2Euler(g)
%     -------------------------------------------------------------------------
%
%     INPUT:
%       g     = <cell> Cell containing M orientation matrices (M 3x3 matrices).
%                      See Randle & Engler, 2000 for more details.
%
%     OUTPUT:
%       euler = <double>  M Euler angles triplets, as row vectors (Mx3 matrix).
%                         Output angles are in degrees and only positive.
%
%     Version 001 15-06-2012 by YGuilhem
%       Vectorize function to process quickly lots of orientations matrices
%
%     Version 001 15-06-2012 by YGuilhem

% Check size of g matrix
if any(size(g{1}) ~= 3)
    gtError('RotMat:wrong_matrix_size', 'Given matrices size is not 3x3!');
end

% Get number of rotation matrices and allocate Euler angles vectors
M = length(g);

% Set epsilon to avoid singular values
epsilon = 1.e-9;

% Convert cell into array of matrices
g_vec = cat(3, g{:});

% Find singular and regular cases
singular = abs(g_vec(3, 3, :) - 1) < epsilon;
regular  = ~singular;

phi1 =        double(singular).*atan2(g_vec(1, 2, :),  g_vec(1, 1, :));
phi1 = phi1 + double(regular ).*atan2(g_vec(3, 1, :), -g_vec(3, 2, :));

phi  = double(regular).*acos(g_vec(3, 3, :));

phi2 =        double(singular).*phi1;
phi2 = phi2 + double(regular ).*atan2(g_vec(1, 3, :), g_vec(2, 3, :));

% Reshape Euler angles vectors
phi1 = reshape(phi1, [M 1]);
phi  = reshape(phi , [M 1]);
phi2 = reshape(phi2, [M 1]);

% Gather all Euler angles in a matrix
euler = rad2deg([ phi1 phi phi2 ]);

% Shift to positive angles range
euler = mod(euler, 360.);

end % end of function
