function r = gtMathsRotMat2Rod(g)
% GTMATHSROTMAT2EULER  Convert orientation matrices to Rodrigues vectors.
%
%     r = gtMathsRotMat2Rod(g)
%     -------------------------------------------------------------------------
%
%     INPUT:
%       g = <cell>    Cell containing M orientation matrices (3x3 matrices).
%                     See Randle & Engler, 2000 for more details.
%
%     OUTPUT:
%       r = <double>  M Rodrigues vectors, as row vectors (Mx3 matrix).
%
%     Version 002 17-10-2012 by YGuilhem
%       Vectorize function to process quickly lots of orientation matrices
%
%     Version 001 15-06-2012 by YGuilhem

% Check size of g matrices
if iscell(g) && all(size(g{1}(:, :)) == [3 3])
    % Convert cell into array of matrices
    g_vec = cat(3, g{:});
elseif all(size(g) == [3 3])
    % Only one g matrix
    g_vec = cat(3, g(:, :));
else
    gtError('RotMat:wrong_matrix_size', 'Given matrices size is not 3x3!');
end

% Set epsilon to avoid singular values
epsilon = 1.e-9;

% Compute orientation matrices traces
g_trace = g_vec(1, 1, :) + g_vec(2, 2, :) + g_vec(3, 3, :);

% Find regular cases and prepare multiplier (null for non regular cases)
regular = abs(g_trace + 1) >= epsilon;
multiplier = double(regular)./(g_trace(:, :, :) + 1);

r = [ g_vec(2, 3, :) - g_vec(3, 2, :) , ...
      g_vec(3, 1, :) - g_vec(1, 3, :) , ...
      g_vec(1, 2, :) - g_vec(2, 1, :) ] .* multiplier(:, [1 1 1], :);

% Reshape resulting matrix 
r = permute(r, [3 2 1]);

end % end of function
