#!/usr/bin/python

import fnmatch
import os
import sys
import subprocess
import string
import re

from xml_dct_io import DCTConf, DCTCheck, DCTOutput

class MexBuilder(object):
    
    file_extensions = [ '*.c', '*.cxx', '*.cpp' ]

    def __init__(self, compiler_path,
                 indir = os.getcwd(),
                 outdir = os.path.join(os.getcwd(), 'bin', 'mex'),
                 extraArgs = []):
        self.indir = indir
        self.outdir = outdir
        self.compiler_path = compiler_path
        self.c_files = { }
        self.mex_info = None
        try:
            self.mex_info = DCTConf().getMexFiles()
        except:
            print("Error: Couldn't get mex files info. Using plain config.")

    def printHelp(self):
        print("\"%s\" builds your mex files in the DCT dir: \"%s\"" %
              (__file__, self.indir) )
        print("Usage: call it from within the directory of DCT. Be aware that" +
              " the DCT environment variable should be adjusted accordingly")
        print(" Options:")
        print("  -h | --help : to show this help")
        print("  -force-compile : to force the compilation even if not needed")
        print("  -v | --verbose : to show more detailed output")
        print("  -c <filename> : to compile only a specific file (wildcards also apply)")

    def findMexs(self):
        """
        def findMexs():

        Finds the mex files (well, the C sources) and saves the files into the object
        """
        print('Finding mex files, from: ' + self.indir)
        for root, dir_name, file_names in os.walk(self.indir):
            for file_extension in self.file_extensions:
                for file_name in file_names:
                    if fnmatch.fnmatch(file_name, file_extension):
                        filepath = os.path.join(root, file_name)
                        self.c_files[file_name] = filepath
        print("Done.\n\nThese files were found:")
        for file_name in self.c_files:
            print('File: ' + file_name + ' ( ' + self.c_files[file_name] + ' )')
        print('')

    def _buildMakeCmd(self, cmd, includes, lib_paths, libs, defines, verbose):
        if verbose is True:
            cmd.append('-v')
        for include in includes:
            cmd.append("-I" + include)
        for lib_path in lib_paths:
            cmd.append("-L" + lib_path)
        for lib in libs:
            cmd.append("-l" + lib)
        for define in defines:
            cmd.append("-D" + define)
        print('\n  file: ' + cmd[1] + ' (cmd: ' + string.join(cmd, ' ') + ' )')
        subprocess.call(cmd)

    def _compileFile(self, file_name, verbose):
        """
        [Internal] def _compileFile(file_name, verbose):

        Tries to compile the given file_name.
        """
        file_path = self.c_files[file_name]
        cmd = [ self.compiler_path, file_path, '-outdir', self.outdir ]

        std_include = [ os.path.join(os.getcwd(), 'zUtil_Cxx', 'include') ]

        if self.mex_info is None:
            # No info at all into the xml file
            print("\nWarning: Mex info not loaded.")
            self._buildMakeCmd(cmd, std_include, [], [], [], verbose)
        else:
            if self.mex_info.isExcluded(file_path) is False:
                props = self.mex_info.getFileProperties(file_path)
                if props is not None:
                    includes = std_include + props.get("includes")
                    self._buildMakeCmd(cmd, includes,
                                       props.get("lib_paths"),
                                       props.get("libs"),
                                       props.get("defines"), verbose)
                else:
                    # info found, but not about this file, so let's use standard params
                    print("\nWarning: No Mex info on file: '" + file_name + "'")
                    self._buildMakeCmd(cmd, std_include, [], [], [], verbose)

    def compileFuncs(self, mfiles_to_consider, force_compile, verbose):
        """
        def compileFuncs(mfiles_to_consider, force_compile, verbose):

        Compiles the mex files into the list.
         - mfiles_to_consider: list mex files to compile (if empty, will compile all)
         - force_compile: if true, compiles even if the file is recent
         - verbose: generates a lot of output
        """
        listOfErrors = [];

        if not os.path.exists(self.outdir):
            print('Creating mex directory')
            os.makedirs(self.outdir)
        print('Compiling mex files:')
        if ( len(mfiles_to_consider) is 0
             or ( len(mfiles_to_consider) is 1 and mfiles_to_consider[0] == 'all' ) ):
            # No preferences, so let's compile everything
            for file_name in self.c_files:
                self._compileFile(file_name, verbose)
        else:
            # Specific files
            for file_name in mfiles_to_consider:
                try:
                    self._compileFile(file_name, verbose)
                except KeyError as exc:
                    # Not found in the list, se let's check if it is a regular expression
                    try:
                        found = False
                        for cfile in self.c_files:
                            if re.search(file_name, cfile):
                                self._compileFile(cfile, verbose)
                                found = True
                        if found is not True:
                            print("Nothing found!")
                            raise exc
                    except:
                        errorMsg = 'ERROR. File not found: ' + file_name
                        print(errorMsg)
                        print(exc)
                        listOfErrors.append(errorMsg);
        print('Done.')
        if len(listOfErrors) is not 0:
            print('Errors happened during compilation:')
            for errorMsg in listOfErrors:
                print('- %s' % errorMsg)

if __name__=="__main__":
    try:
        DCTCheck().checkCwdWithDCTDir()
        MexPath = DCTConf().getMexPath()
    except BaseException as ex:
        DCTOutput().printError(ex.args)
        exit(1);

    skip_next = False
    force_compile = False
    verbose = False
    mfiles_to_consider = [ ]
    unclassified_args = [ ]
    try:
        for index in range(len(sys.argv)):
            if skip_next is False:
                if sys.argv[index] == "-force-compile":
                    force_compile = True
                elif sys.argv[index] == "--verbose" or sys.argv[index] == "-v":
                    verbose = True
                elif sys.argv[index] == "-c":
                    skip_next = True
                    if len(sys.argv) > (index+1):
                        mfiles_to_consider.append(sys.argv[index+1])
                    else:
                        raise ValueError("Not enough arguments to '-c' option")
                elif sys.argv[index] == "--help" or sys.argv[index] == "-h":
                    raise ValueError("")
                else:
                    unclassified_args.append(sys.argv[index])
            else:
                skip_next = False

        builder = MexBuilder(MexPath, extraArgs = unclassified_args)
        builder.findMexs()
        builder.compileFuncs(mfiles_to_consider, force_compile, verbose)
    except ValueError as ex:
        if (len(ex.args) is not 0) and (ex.args[0] is not ""):
            DCTOutput().printError(ex.args)
        MexBuilder(MexPath).printHelp();

