function mesh = gtVTKMeshReader(filename, varargin)
% GTVTKMESHREADER  Read mesh (polydata) from a VTK file.
%
%   mesh = gtVTKMeshReader(filename, varargin)
%   --------------------------------------------------------------------------
%
%     INPUT:
%       filename = <string>  Path to the input VTK file
%
%     OUTPUT:
%       mesh     = <struct>  Mesh stored in a structure, composed of 2 fields :
%                            - vertices: Nv x 3 array -> vertices coordinates
%                            - faces: Nf x 3 array -> face vertices indices
%
%     Version 001 20-10-2012 by YGuilhem yoann.guilhem@esrf.fr

%% Guessing output directory, filename and extension
[fpath, fname, fext] = fileparts(filename);
if isempty(fext)
    fext = '.vtk';
end
% Trick to handle ~ path in unix
if ~isempty(fpath) && isunix && fpath(1)=='~'
    fpath(1) = '';
    fpath = fullfile(getenv('HOME'), fpath);
end
filename = fullfile(fpath, [fname fext]);

fid = fopen(filename, 'r');
if fid ==-1
    gtError('gtVTKMeshReader:bad_input_file', 'Can''t open the file.');
end

file.desc = fgets(fid);   % -1 if eof
if ~strcmp(file.desc(3:5), 'vtk')
    gtError('gtVTKMeshReader:bad_input_file', 'Not a valid VTK file.');
end

%% Read header, mode and dataset
file.header  = fgets(fid);
str          = fgets(fid);
file.mode = sscanf(str, '%s', 1);
binary = false;
if strcmp(file.mode, 'BINARY')
    binary = true;
end
str = fgets(fid);
file.dataset = sscanf(str, 'DATASET %s', 1);
if ~strcmp(file.dataset, 'POLYDATA')
    gtError('gtVTKMeshReader:wrong_content', ...
        'This vtk file does not contain PolyData dataset!');
end

%% Read number of vertices and coordinate datatype
str = fgets(fid);
nVert = sscanf(str, 'POINTS %d', 1);
file.nVertices = nVert;
str2 = ['POINTS ' num2str(nVert) ' %s'];
coordType = sscanf(str, str2);
if binary
    switch coordType
    case {'float', 'double'}
        coordFMT = coordType;
    otherwise
        gtError('gtVTKMeshReader:wrong_coord_type', ...
            ['cannot handle coordinate with type: ' coordType]);
    end
else
    switch coordType
    case 'float'
        coordFMT = '%f %f %f';
    case 'double'
        coordFMT = '%f %f %f';
    otherwise
        true;
    end
end

%% Read vertices
nCoord = 3*nVert;
if binary
    coord = fread(fid, nCoord, ['3*' coordFMT '=>' coordFMT], 0 , 'ieee-be');
else
    [coord, sz] = fscanf(fid, '%f %f %f', nCoord);
    if sz ~= nCoord
        warning('gtVTKMeshReader:vertice_warning', ...
            'Problem in reading vertices.');
    end
end
mesh.vertices = reshape(coord, 3, nVert)';

%% Read polygons
str = fgets(fid);
while str(1) ~= 'P' %&& str(1) ~= 'V'
    str = fgets(fid);    
end

if str(1) == 'P'
    numbers = sscanf(str, '%*s %d %d', 2);
    nFace = numbers(1);
    nValue = numbers(2);
    nNode = 3*nFace;
    if binary
        nbytes = 4;
        fseek(fid, nbytes, 0);
        connectivity = fread(fid, nValue, '3*int32=>uint32', nbytes, 'ieee-be');
    else
        [connectivity, cnt] = fscanf(fid, '%*d %d %d %d\n', nNode);
        if cnt ~= nNode
            warning('gtVTKMeshReader:face_warning', ...
                'Problem in reading faces.');
        end
        connectivity = uint32(connectivity);
        class(connectivity)
    end
    mesh.faces = reshape(connectivity+1, 3, nFace)';
else
    mesh.faces = 0;
end

% read vertex ids
%if info == 'V'
%    nv = sscanf(str, '%*s %d %*s', 1);

%    [A, cnt] = fscanf(fid, '%d %d \n', 2*nv);
%    if cnt~=2*nv
%        warning('gtVTKMeshReader:vertex_warning', ...
%            'Problem in reading vertex.');
%    end

%    A = reshape(A, 2, cnt/2);
%    faces = repmat(A(2, :)+1, 3, 1);
%end

%if str(1) ~= 'P' %&& str(1) ~= 'V'
%    faces = 0;
%end

%% Close the file end terminate
fclose(fid);

end % end of function
