function gtVTKMeshWriter(mesh, filename, varargin)
% GTVTKMESHWRITER  Write a mesh to VTK file (polydata) with fields or color maps.
%
%   gtVTKMeshWriter(mesh, filename, varargin)
%   --------------------------------------------------------------------------
%
%     INPUT:
%       mesh     = <struct>  Mesh stored in a structure, composed of 2 fields :
%                            - vertices: Nv x 3 array -> vertices coordinates
%                            - faces: Nf x 3 array -> face vertices indices
%       filename = <string>  Path to the output VTK file
%
%     OPTIONAL INPUT:
%       'binary'    = <bool>   Write output in binary format (true by default)
%       'coordType' = <string> Output type for coordinates ('float', 'double')
%       'facefield' = <struct> Face field struct, composed of following fields:
%                              - name    = <string> Name of the field
%                              - data    = <any>    Data array
%                              - cmap    = <cmap>   Colormap         (optional)
%                              - opacity = <double> Color opacity    (optional)
%                              - type    = <string> Field type   (only scalars)
%       'facecolor' = <struct> Face color struct, composed of following fields:
%                              - name    = <string> Name of the field
%                              - data    = <any>    Color array 
%                              - opacity = <double> Color opacity    (optional)
%
%     Version 001 20-10-2012 by YGuilhem yoann.guilhem@esrf.fr

par.header = 'VTK mesh created by matlabDCT';
par.verbose = false;
par.binary = true;
par.coordType = 'float';
par.facefield = [];
par.facecolor = [];
par = parse_pv_pairs(par, varargin);

%% Check coordinate data type
switch par.coordType
case {'float', 'double'}
    coordFMT = '%f %f %f\n';
otherwise
    gtError('gtVTKMeshWriter:wrong_coord_type', ...
        ['Unsupported coordinates datatype: ' par.coordType]);
end

%% Get file parts and check extension
[fpath, fname, fext] = fileparts(filename);
if isempty(fext)
    fext = '.vtk';
    filename = fullfile(fpath, [fname fext]);
elseif ~strcmp(fext, '.vtk')
    gtError('gtVTKMeshWriter:wrong_file_extension', ...
        'Output file extension should be ''.vtk''!');
end

%% Open file for writing
fid = fopen(filename, 'w');
if fid ==-1
    gtError('gtVTKMeshWriter:bad_output_file', 'Can''t open the file.');
end

%% Header
fprintf(fid, '# vtk DataFile Version 3.0\n');
fprintf(fid, '%s\n', par.header);
if par.binary 
    fprintf(fid, 'BINARY\n');
else
    fprintf(fid, 'ASCII\n');
end
fprintf(fid, 'DATASET POLYDATA\n');

%% Points / Vertices
fprintf(fid, 'POINTS %d %s\n', size(mesh.vertices, 1), par.coordType);
if par.binary
    count = fwrite(fid, mesh.vertices', par.coordType, 0, 'ieee-be');
    if count ~= numel(mesh.vertices)
        warning('gtVTKMeshWriter:vertice_warning', ...
            'Problem in writing vertices.');
    end
    fprintf(fid, '\n');
else
    fprintf(fid, coordFMT, mesh.vertices');
end

%% Polygons / Faces / Cells
nFaces = size(mesh.faces, 1);
nValues = 4*nFaces;
fprintf(fid, 'POLYGONS %d %d\n', nFaces, nValues);
output = [3*ones(1, nFaces) ; mesh.faces'-1];
if par.binary
    count = fwrite(fid, output, 'uint32', 0, 'ieee-be');
    if count ~= nValues
        warning('gtVTKMeshWriter:face_warning', 'Problem in writing faces.');
    end
    fprintf(fid, '\n');
else
    fprintf(fid, '%d %d %d %d\n', output);
end

%% Face field
if ~isempty(par.facefield)
    for ifield=1:length(par.facefield)
        fField = par.facefield(ifield);
        % Check opacity
        if ~isfield(fField, 'opacity') || isempty(fField.opacity)
            fField.opacity = 1;
        end
        % Check field type
        if ~isfield(fField, 'fieldtype') || isempty(fField.fieldtype)
            fField.fieldtype = 'scalars';
        elseif ~strcmpi(fField.fieldtype, 'scalars')
            gtError('gtVTKMeshWriter:wrong_field_type', ...
                'Sorry, only scalar fields are implemented right now');
        end

        inputType = class(fField.data);
        [vtkType, outType, outFMT] = gtVTKConvertTypeMatlab2VTK(inputType);
        fprintf(fid, 'CELL_DATA %d\n', nFaces);
        fprintf(fid, '%s %s %s 1\n', upper(fField.fieldtype), fField.name, vtkType);
        % Link to tookup table
        if ~isfield(fField, 'cmap') || isempty(fField.cmap)
            fprintf(fid, 'LOOKUP_TABLE default\n');
        else
            fprintf(fid, 'LOOKUP_TABLE fftable%d\n', ifield);
        end
        % Field values
        if par.binary
            count = fwrite(fid, fField.data, outType, 0, 'ieee-be');
            if count ~= numel(fField.data)
                warning('gtVTKMeshWriter:face_warning', ...
                    ['Problem in writing field of face field ''' fField.name '']);
            end
            fprintf(fid, '\n');
        else
            fprintf(fid, [outFMT '\n'], fField.data);
        end
        % Lookup table
        if isfield(fField, 'cmap') && ~isempty(fField.cmap)
            fprintf(fid, 'LOOKUP_TABLE fftable%d %d\n', ifield, size(fField.cmap, 1));
            output = [fField.cmap' ; fField.opacity*ones(1, size(fField.cmap, 1))];
            if par.binary
                count = fwrite(fid, uint8(255*output), 'uchar', 0, 'ieee-be');
                if count ~= numel(output)
                    warning('gtVTKMeshWriter:lookup_table_warning', ...
                        ['Problem in writing lookup table of face field ''' fField.name '']);
                end
                fprintf(fid, '\n');
            else
                fprintf(fid, '%f %f %f %f\n', output);
            end
        end
    end
end    

%% Face color
if ~isempty(par.facecolor)
    for ifield=1:length(par.facecolor)
        fField = par.facecolor(ifield);
        % Check opacity
        if ~isfield(fField, 'opacity') || isempty(fField.opacity)
            output = fField.data';
            outFMT = '%f %f %f\n';
            nColorComp = 3;
            disp('no opacity');
        else
            output = [fField.data' ; fField.opacity*ones(1, size(fField.data, 1))];
            outFMT = '%f %f %f %f\n';
            nColorComp = 4;
        end
        % Field values
        fprintf(fid, 'CELL_DATA %d\n', nFaces);
        fprintf(fid, 'COLOR_SCALARS %s %d\n', fField.name, nColorComp);
        if par.binary
            count = fwrite(fid, uint8(255*output), 'uchar', 0, 'ieee-be');
            if count ~= numel(output)
                warning('gtVTKMeshWriter:face_field_warning', ...
                    ['Problem in writing field of face field ''' fField.name '']);
            end
            fprintf(fid, '\n');
        else
            fprintf(fid, outFMT, output);
        end
    end
end    
    
%% Close the file end terminate
fclose(fid);

end % end of function
