function gtVTKStructGridWriter(vol, filename, varargin)
% GTVTKSTRUCTGRIDWRITER  Write a 3D volume in a VTK file.
%
%   gtVTKStructGridWriter(vol, filename, varargin)
%   --------------------------------------------------------------------------
%
%     INPUT:
%       vol         = <3Dimage> Input 3D volume 
%       filename    = <string>  Path to the output VTK file
%
%     OPTIONAL INPUT (varargin as a list of pairs, see parse_pv_pairs.m):
%       'binary'    = <bool>    Write output in binary format (true by default)
%       'spacing'   = <double>  Vector defining spacing in each space dimension
%                               ([1 1 1] by default)
%       'origin'    = <double>  Vector defining volume origin
%                               ([0 0 0] by default)
%       'header'    = <string>  VTK file header description
%       'datatype'  = <string>  Force data type to consider for input volume
%       'cmap'      = <cmap>    Color map for volume data for vizualization
%       'opacity'   = <double>  Opacity for vizualization (1 by default)
%       'verbose'   = <bool>    Verbose mode
%
%     Version 001 24-11-2012 by YGuilhem yoann.guilhem@esrf.fr

par.spacing = ones(1, 3);
par.origin  = zeros(1, 3);
par.dataname = 'Values';
par.header   = 'VTK structured volume created by matlabDCT';
par.datatype = ''; 
par.cmap = [];
par.opacity = 1;
par.verbose = false;
par.binary = true;
par = parse_pv_pairs(par, varargin);

if iscell(vol)
    error('Not implemented: only one volume can be handled by now!');
else 
    [nx, ny, nz] = size(vol);
end
ntot = nx*ny*nz;

if ~isempty(par.cmap) && size(par.cmap, 1) < max(max(max(vol)))
    warning('gtVTKStructGridWriter:cmap_warning', ...
        'You may have not enough values in your cmap...');
end

%% Guess input & output datatype
if isempty(par.datatype)
    par.datatype = class(vol);
end

[outputTypeVTK, outputType, outputFMT] = gtVTKConvertTypeMatlab2VTK(par.datatype);

if par.verbose
    disp(['Spacing       : ' num2str(par.spacing)]);
    disp(['Origin        : ' num2str(par.origin)]);
    disp(['Color map     : ' num2str(par.cmap)]);
    disp(['Dataname      : ' par.dataname]);
    disp(['Header        : ' par.header]);
    disp(['OutputType    : ' outputType]);
    disp(['OutputTypeVTK : ' outputTypeVTK]);
end

%% Open the file
fid = fopen(filename, 'w');
if fid ==-1
    gtError('gtVTKStructGridWriter:bad_output_file', 'Can''t open the file.');
end

%% Write ASCII file header
fprintf(fid, '# vtk DataFile Version 3.0\n');
fprintf(fid, '%s\n', par.header);
if par.binary 
    fprintf(fid, 'BINARY\n');
else
    fprintf(fid, 'ASCII\n');
end
fprintf(fid, 'DATASET STRUCTURED_POINTS\n');
fprintf(fid, 'DIMENSIONS %d %d %d\n', nx, ny, nz);
fprintf(fid, 'ORIGIN  %4.3f   %4.3f  %4.3f\n', par.origin);
fprintf(fid, 'SPACING %4.3f   %4.3f  %4.3f\n', par.spacing);

fprintf(fid, 'POINT_DATA %d\n', ntot);
fprintf(fid, 'SCALARS %s %s\n', par.dataname, outputTypeVTK);

if ~isempty(par.cmap)
    fprintf(fid, 'LOOKUP_TABLE map\n');
else
    fprintf(fid, 'LOOKUP_TABLE default\n');
end

%% Write volume data
if par.binary 
    count = fwrite(fid, reshape(vol, 1, ntot), par.datatype, 0, 'ieee-be');
    if count ~= numel(vol)
        warning('gtVTKStructGridWriter:structgrid_volume_warning', ...
            'Problem in writing structured grid volume.');
    end
    fprintf(fid, '\n');
else
    fprintf(fid, [outputFMT '\n'], reshape(vol, 1, ntot));
end

%% Write lookup table if needed
if ~isempty(par.cmap)
    mapSize = size(par.cmap);
    if mapSize(2) == 3
        output = [par.cmap'; par.opacity*ones(1, mapSize(1))];
    elseif mapSize(2) == 4
        output = par.cmap';
    else
        gtError('gtVTKStructGridWriter:wrong_cmap_size', ...
            'Color map sould be N row vector of size 3 or 4!');
    end
    fprintf(fid, 'LOOKUP_TABLE map %d\n', mapSize(1));
    if par.binary
        count = fwrite(fid, uint8(225*output), 'uchar', 0, 'ieee-be');
        if count ~= numel(output)
            warning('gtVTKMeshWriter:cmap_warning', ...
                'Problem in writing color map!');
        end
        fprintf(fid, '\n');
    else
        fprintf(fid, '%f %f %f %f\n', output);
    end
end

%% Close file and terminate
fclose(fid);

end % end of function
